// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESCL_OS_H__
#define __ESCL_OS_H__

#include "escl_platform.h"

/************************************************************************
 *  Data Type Definitions
 *************************************************************************/
#ifndef INVALID_VALUE
#define INVALID_VALUE -1
#endif /*INVALID_VALUE*/

#ifndef DEF_LOCAL_PORT
#define DEF_LOCAL_PORT 6779
#endif /*DEF_LOCAL_PORT*/

#ifndef DEF_REMOTE_PORT
#define DEF_REMOTE_PORT 6778
#endif /*DEF_REMOTE_PORT*/

#ifndef IS_NOT_ALIGNED
#define IS_NOT_ALIGNED(addr, n) ((uint32_t)(addr) & ((n)-1))
#endif /* IS_NOT_ALIGNED*/

#ifndef RET_FAILED
#define RET_FAILED -1
#endif /*RET_FAILED*/

#ifndef ERR_SEMAPHORE
#define ERR_SEMAPHORE -111
#endif /*ERR_SEMAPHORE*/

#ifndef ERR_TIMEOUT
#define ERR_TIMEOUT -112
#endif /*ERR_TIMEOUT*/

#ifndef ERR_THREAD
#define ERR_THREAD -113
#endif /*ERR_THREAD*/

#ifndef ERR_CONDITION
#define ERR_CONDITION -114
#endif /*ERR_CONDITION*/

#ifndef ERR_MUTEX
#define ERR_MUTEX -115
#endif /*ERR_MUTEX*/

#ifndef ALIGNED
#define ALIGNED(addr, n) (((uint32_t)(addr) + ((n)-1)) & (~((n)-1)))
#define ALIGNED4(addr) ALIGNED(addr, 4)
#define ALIGNED8(addr) ALIGNED(addr, 8)
#define ALIGNED16(addr) ALIGNED(addr, 16)
#define ALIGNED32(addr) ALIGNED(addr, 32)
#endif /*ALIGNED*/

#ifndef ALIGN_DOWN
#define ALIGN_DOWN(length, type) ((uint32_t)(length) & ~(sizeof(type) - 1))
#endif /*ALIGN_DOWN*/

#ifndef ALIGN_UP
#define ALIGN_UP(length, type) (ALIGN_DOWN(((uint32_t)(length) + sizeof(type) - 1), type))
#endif /*ALIGN_UP*/

#ifndef ALIGN_DOWN_POINTER
#define ALIGN_DOWN_POINTER(address, size) ((uint32_t)((uint32_t)(address) & ~((uint32_t)(size)-1)))
#endif /*ALIGN_DOWN_POINTER*/

#ifndef ALIGN_UP_POINTER
#define ALIGN_UP_POINTER(address, size) (ALIGN_DOWN_POINTER(((uint32_t *)(address) + (size - 1)), size))
#endif /*ALIGN_UP_POINTER*/

#ifndef ROUND_DOWN
#define ROUND_DOWN(addr, boundary) ((uint32_t)(addr) & ~((boundary)-1))
#endif  // ROUND_DOWN

#ifndef ROUND_UP
#define ROUND_UP(addr, boundary) (((uint32_t)(addr) + (boundary)-1) & ~((boundary)-1))
#endif  // ROUND_UP

#ifndef __SWAB16
#define __SWAB16(x) ((((x)&0xFF) << 8) | (((x) >> 8) & 0xFF))
#endif /*__SWAB16*/

#ifndef __SWAB32
#define __SWAB32(x) ((((x)&0xff) << 24) | (((x)&0xff00) << 8) | (((x) >> 8) & 0xff00) | (((x) >> 24) & 0xff))
#endif /* __SWAB32 */

#if (ESCL_BYTEORDER) == (ESCL_BIG_ENDIAN)
#define Htonl(a) (a)
#define Ntohl(a) (a)
#define Htons(a) (a)
#define Ntohs(a) (a)
#else
#define Htonl(a) __SWAB32(a)
#define Ntohl(a) __SWAB32(a)
#define Htons(a) __SWAB16(a)
#define Ntohs(a) __SWAB16(a)
#endif /*ESCL_BYTEORDER == ESCL_BIG_ENDIAN*/

#define bit_reverse_u32(x)                                                                                             \
    ((x) = (x) >> 16 | (0x0000ffff & (x)) << 16, (x) = (0xff00ff00 & (x)) >> 8 | (0x00ff00ff & (x)) << 8,              \
     (x) = (0xf0f0f0f0 & (x)) >> 4 | (0x0f0f0f0f & (x)) << 4, (x) = (0xcccccccc & (x)) >> 2 | (0x33333333 & (x)) << 2, \
     (x) = (0xaaaaaaaa & (x)) >> 1 | (0x55555555 & (x)) << 1)
#define bit_reverse_u8(x)                                                                      \
    ((x) = (0xf0 & (x)) >> 4 | (0x0f & (x)) << 4, (x) = (0xcc & (x)) >> 2 | (0x33 & (x)) << 2, \
     (x) = (0xaa & (x)) >> 1 | (0x55 & (x)) << 1)

#if defined(__GNUC__)
#define atomic_inc(ptr) __sync_add_and_fetch((ptr), 1)
#define PACK(__Declaration__) __Declaration__ __attribute__((__packed__))
#ifndef __packed
#define __packed __attribute__((packed))  // byte aligned.
#endif                                    /* __packed */
#ifndef __unused
#define __unused __attribute__((unused))  // Avoids warning for unused variable and function.
#endif                                    /* __unused */
#ifndef __align_8
#define __align_8 __attribute__((aligned(8)))  // 8 bytes aligned.
#endif                                         /*__align_8 */
#ifndef __align_16
#define __align_16 __attribute__((aligned(16)))  // 16 bytes aligned.
#endif                                           /*__align_16 */
#ifndef __align_32
#define __align_32 __attribute__((aligned(32)))  // 32 bytes aligned.
#endif                                           /*__align_32 */
#ifndef __weak
#define __weak __attribute__((weak))  // Weak symbol.
#endif                                /*__weak*/
#ifndef __format_printf
#define __format_printf(n, m) __attribute__((format(printf, n, m)))
#endif  //__format_printf
#ifndef __format_scanf
#define __format_scanf(n, m) __attribute__((format(scanf, n, m)))
#endif  //__format_scanf
#ifndef __calltype
#if defined(__SDE_MIPS__)
#define __calltype
#else                                      //__SDE_MIPS__
#define __calltype __attribute__((cdecl))  // Call convention.
#endif                                     //__SDE_MIPS__
#endif                                     /*__calltype */
#define __ESCL_PACKED_PUSH(n)
#define __ESCL_PACKED_POP()

#else /*__GNUC__ is not defined */
#define atomic_inc(ptr) InterlockedIncrement((volatile LONG *)(ptr))
#define PACK(__Declaration__) __pragma(pack(push, 1)) __Declaration__ __pragma(pack(pop))

#ifndef __packed
#define __packed  //#pragma   pack(1)
#endif            /* __packed */
#ifndef __unused
#define __unused
#endif /* __unused */
#ifndef __align_8
#define __align_8  // #pragma   pack(8)
#endif             /*__align_8 */
#ifndef __align_16
#define __align_16
#endif /*__align_16 */
#ifndef __align_32
#define __align_32
#endif /*__align_32 */
#ifndef __format_scanf
#define __format_scanf(n, m)
#endif  //__format_scanf
#ifndef __format_printf
#define __format_printf(n, m)
#endif  //__format_printf
#ifndef __weak
#define __weak
#endif /*__weak*/
#ifndef __calltype
#define __calltype __stdcall
#endif /*__calltype */

#if !defined(LINUX) && !defined(__linux) && !defined(__CYGWIN__) && !defined(WIN32) && !defined(WINDOWS)
#define __ESCL_PACKED_PUSH(n) #pragma pack(push, n)
#define __ESCL_PACKED_POP() #pragma pack(pop)
#endif
#endif /*__GNUC__*/

#if defined(WIN32) || defined(WINDOWS)
#define ESCL_CALLTYPE
#else /*WIN32 || WINDOWS*/
#define ESCL_CALLTYPE __attribute__((cdecl))
#endif /*WIN32 || WINDOWS*/

typedef int escl_err_t;          // Type of return value.
typedef const char *escl_dbl_t;  // For debug message levels.
typedef int escl_handle_t;

#if defined(__cplusplus)
extern "C" {
#endif /*!__cplusplus*/

#if defined(WIN32) || defined(WINDOWS)
typedef DWORD pid_t;
#endif /*WIN32 || WINDOWS*/

#if defined(WIN32) || defined(WINDOWS)

#if (!defined(_WIN32_WINNT) || _WIN32_WINNT < 0x0400)
#pragma message "Unsupported platform, Windows NT 4.0 or later required"
#endif  //_WIN32_WINNT

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#define inline __inline
#endif

// Priority class of a thread.
#define ESCL_THREAD_PRIORITY_LOWEST (THREAD_PRIORITY_LOWEST)
#define ESCL_THREAD_PRIORITY_BELOW_NORMAL (THREAD_PRIORITY_BELOW_NORMAL)
// Normal priority for the priority class.
#define ESCL_THREAD_PRIORITY_NORMAL (THREAD_PRIORITY_NORMAL)
#define ESCL_THREAD_PRIORITY_ABOVE_NORMAL (THREAD_PRIORITY_ABOVE_NORMAL)
#define ESCL_THREAD_PRIORITY_CRITICAL (THREAD_PRIORITY_TIME_CRITICAL)
// The highest priority of thread.
#define ESCL_THREAD_PRIORITY_HIGHEST (THREAD_PRIORITY_HIGHEST)

// Normally stack size of a thread in Win32.
#define ESCL_THREAD_STACK_SIZE (1024 * 1024)
// Type definition of thread handle(WIN32).
typedef HANDLE escl_thread_t;
// Type definition of spin lock(WIN32).
typedef CRITICAL_SECTION escl_spinlock_t;
// Type definition of mutex lock(WIN32).
typedef HANDLE escl_mutex_t;
// Type definition of thread's return value(WIN32).
typedef unsigned escl_thread_ret_t;
// Type definition of thread's argument(WIN32).
typedef void *escl_thread_data_t;
// Type definition of Semaphore.
typedef HANDLE escl_semaphore_t;

typedef DWORD escl_thread_key_t;

enum {
    SIGNAL = 0,
    BROADCAST = 1,
    MAX_EVENTS = 2
};

#define __NO_LOCAL_COND_VAR__
#if defined(__NO_LOCAL_COND_VAR__)
typedef struct __ESCL_COND_T {
    HANDLE events_[MAX_EVENTS];
    // Signal and broadcast event HANDLEs.
} escl_cond_t;
#else /*__NO_LOCAL_COND_VAR__*/
typedef CONDITION_VARIABLE escl_cond_t;
#define PTHREAD_COND_INITIALIZER \
    {                            \
        0                        \
    }
#endif /*__NO_LOCAL_COND_VAR__*/

#define myGetPid() GetCurrentProcessId()
#define vmalloc(sz) malloc(sz)
#define vfree(p) free(p)
#ifndef GFP_KERNEL
#define GFP_KERNEL 0
#endif /*GFP_KERNEL*/
#define kmalloc(sz, flag) malloc(sz)
#define kfree(p) free(p)

// Type definition of thread handler.
typedef escl_thread_ret_t(__calltype *escl_thread_func_t)(escl_thread_data_t arg);

/************************************************************************
 *
 *                      int escl_thread_create
 *                         (
 *                          escl_thread_t        *tid,
 *                          escl_thread_func_t    func,
 *                          escl_thread_data_t    arg,
 *                          unsigned             stack_size
 *                          )
 *  Description :
 *  -------------
 *   The escl_thread_create() function is used to create a new thread, with
 *   stack size specified by stack_size(The default size was defined by
 *   ESCL_THREAD_STACK_SIZE), Upon successful completion, escl_thread_create()
 *   will store the ID of the created thread in the location specified by tid.
 *
 *   Upon its creation, the thread executes func, with arg as its
 *   sole argument.  If start_routine returns, the effect is as if there was
 *   an implicit call return instruction,
 *
 *  Parameter :
 *  -----------
 *  tid        :    returned handle to the thread,
 *                  This pointer indicates where the new thread id should be put into.
 *  func       :    Routine that begins execution of the thread.
 *  arg        :    data value passed to the thread entry routine.
 *  stack_size :    Size, In bytes, of the stack for the thread.
 *  ---------------
 *  If successful, the escl_thread_create() function will return zero.
 *  Otherwise, an error number will be returned to indicate the error.
 *
 ************************************************************************/

#define escl_thread_create(tid, func, arg, stack_size) \
    ((*(tid) = (HANDLE)_beginthreadex(NULL, (stack_size), func, (void *)arg, 0, (unsigned *)0)) ? 0 : ERR_THREAD)

/************************************************************************
 *
 *              int escl_thread_join(escl_thread_t tid);
 *  Description :
 *  -------------
 *  The escl_thread_join() function suspends execution of the calling thread
 *  until the target thread terminates, unless the target thread has already
 *��terminated.
 *
 *  Parameter :
 *  -----------
 *   tid      :   Target thread ID.
 *
 *  Return values:
 *  ---------------
 *  If successful,  the escl_thread_join() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/

#define escl_thread_join(tid) ((WaitForSingleObject((tid), INFINITE) == WAIT_OBJECT_0) ? 0 : ERR_THREAD)

/************************************************************************
 *
 *                  escl_thread_t escl_thread_self(void)
 *  Description :
 *  -------------
 *   The escl_thread_self() function shall return the thread ID of
 *   the calling thread.
 *
 *  Parameter :
 *  -----------
 *   N/A
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_self() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
#define escl_thread_self() GetCurrentThread()

/************************************************************************
 *
 *      int escl_thread_setpriority(escl_thread_t tid, const int priority)
 *  Description :
 *  -------------
 *  Set the thread to the specified priority level. The valid ranges for the priority value are
 *  determined by the ESCL_THREAD_PRIORITY_LOWEST and ESCL_THREAD_PRIORITY_HIGHEST.
 *  default priority is determined by ESCL_THREAD_PRIORITY_NORMAL.
 *  eCos  from 1 to 32, where lower values represent higher thread priorities.
 *  Win32 from THREAD_PRIORITY_LOWEST to THREAD_PRIORITY_HIGHEST
 *  linux from 0 to 99.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  priority    :       priority of thread will be set.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_setpriority(tid, priority) (SetThreadPriority((tid), (priority)) ? 0 : ERR_THREAD)

/************************************************************************
 *
 *      int escl_thread_getpriority(escl_thread_t tid)
 *
 *  Description :
 *  -------------
 *  Retrieves the priority value for the specified thread.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  The return value is the thread's priority level.
 ************************************************************************/
#define escl_thread_getpriority(tid) GetThreadPriority(tid)

/************************************************************************
 *
 *                void escl_thread_delay(unsigned msecs)
 *
 *  Description :
 *  -------------
 *  Suspends execution of the calling thread for (at least) msecs microseconds.
 *  The sleep may be lengthened slightly by any system activity or by the time
 *  spent processing the call or by the granularity of system timers.
 *
 *  Parameter :
 *  -----------
 *  msecs         :          time for thread to sleep (in microseconds).
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_delay(msecs) Sleep(msecs)

/************************************************************************
 *
 *                 void escl_thread_suspend(escl_thread_t tid)
 *  Description :
 *  -------------
 *  Postpones the execution of a thread. If a thread is suspended multiple times, the resume
 *  function call must be made for each suspend function call before the thread will execute.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_suspend(tid) SuspendThread(tid)

/************************************************************************
 *
 *               void escl_thread_resume(escl_thread_t tid)
 *  Description :
 *  -------------
 *  Causes a thread to continue execution. This call must be made after a thread is created in
 *  order to start the execution of the thread. Each suspend function call must correspond to a
 *  resume function call before a thread will continue execution.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_resume(tid) ResumeThread(tid)

/************************************************************************
 *
 *                   void escl_thread_yield(void)
 *  Description :
 *  -------------
 *   The escl_thread_yield() function resets the accumulated time slice of the
 *   current thread and then enters the thread scheduler, which resumes execu-
 *   tion of the next thread ready to run.  If no other thread is ready to
 *   run, control returns to the current thread.
 *
 *  Parameter :
 *  -----------
 *   N/A
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_yield() SwitchToThread()

/************************************************************************
 *
 *             void escl_thread_destroy(escl_thread_t tid)
 *  Description :
 *  -------------
 *   Free the allocated stack of a thread. it is necessary for eCos.
 *
 *  Parameter :
 *  -----------
 *   tid      Thread ID.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_destroy(tid) CloseHandle(tid)

/************************************************************************
 *
 *                  void escl_thread_exit(unsigned long ret)
 *  Description :
 *  -------------
 *  terminate the calling thread,
 *
 *
 *  Parameter :
 *  -----------
 *   ret      return value of called thread.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_exit(ret) ExitThread((DWORD)ret)

/************************************************************************
 *
 *                  int escl_thread_cancel(escl_thread_t tid)
 *  Description :
 *  -------------
 *  escl_thread_cancel() is used to cause a thread to exit.
 *  In most system will frees the thread's resources(such as stack)
 *  except eCos. you should call escl_thread_join() to free the allocated
 *  resources of the thread.
 *
 *
 *  Parameter :
 *  -----------
 *   tid      :  Thread ID
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_cancel() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
#define escl_thread_cancel(tid) ((TerminateThread((tid), 0) == 0) ? ERR_THREAD : 0)

extern int escl_thread_cancel_wait(escl_thread_t tid);

/************************************************************************
 *
 *         int escl_mutex_init(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  Creates or opens a named or unnamed mutex object.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/

#define escl_mutex_init(x) ((*(x) = CreateMutex(NULL, FALSE, NULL)) ? 0 : ERR_MUTEX)

/************************************************************************
 *
 *         void escl_mutex_destroy(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_destroy() function frees the resources allocated for
 mutex.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/

#define escl_mutex_destroy(x) CloseHandle(*(x))

/************************************************************************
 *
 *         int escl_mutex_lock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The mutex object referenced by mutex shall be locked by calling escl_mutex_lock().
 *  If the mutex is already locked,
 *  the calling thread shall block until the mutex becomes available.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_lock(x) ((WaitForSingleObject(*(x), INFINITE) == WAIT_OBJECT_0) ? 0 : ERR_MUTEX)

/************************************************************************
 *
 *         int escl_mutex_unlock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_unlock() function shall release the mutex object
 *  referenced by mutex. The manner in which a mutex is released is
 *  dependent upon the OS. If there are threads
 *  blocked on the mutex object referenced by mutex when escl_mutex_unlock() is called,
 *  resulting in the mutex becoming available,
 *  the scheduling policy shall determine which thread shall acquire the mutex.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_unlock(x) ReleaseMutex(*(x))

/************************************************************************
 *
 *         int escl_mutex_trylock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_trylock() function locks mutex.  If the mutex is
 *  already locked, escl_mutex_trylock() will not block waiting for the
 *   mutex, but will return an error condition.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  "true" if the mutex was locked, "false" if the mutex couldn't be locked.
 *
 ************************************************************************/
#define escl_mutex_trylock(x) ((WaitForSingleObject(*(x), 0) == WAIT_OBJECT_0))

/************************************************************************
 *
 *   int escl_thread_key_create(escl_thread_key_t *key, void (*destructor)(void*));
 *
 *  Description :
 *  -------------
 *  The escl_thread_key_create() function shall create a thread-specific data key
 *  visible to all threads in the process. Key values provided by escl_thread_key_create()
 *  are opaque objects used to locate thread-specific data. Although the same key
 *  value may be used by different threads, the values bound to the key by
 *  escl_thread_setspecific() are maintained on a per-thread basis and persist
 *  for the life of the calling thread.
 *  Parameter     :
 *  -----------
 *  key         :    pointer to received the created key.
 *  func        :    An optional destructor function may be associated with each
 *                   key value. At thread exit, if a key value has a non-NULL destructor pointer,
 *                   and the thread has a non-NULL value associated with that key, the value of the
 *                   key is set to NULL, and then the function pointed to is called with the previously
 *                   associated value as its sole argument. The order of destructor calls is unspecified
 *                   if more than one destructor exists for a thread when it exits
 *  Return values :
 *  ---------------
 *  If successful, returns zero; otherwise, an error number shall be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_key_create(key, func) (((*(key) = TlsAlloc()) == TLS_OUT_OF_INDEXES) ? -1 : 0)

/************************************************************************
 *
 *   int escl_thread_key_delete(escl_thread_key_t key);
 *
 *  Description :
 *  -------------
 *  The escl_thread_key_delete() function shall delete a thread-specific data
 *  key previously returned by escl_thread_key_create(). The thread-specific data
 * values associated with key need not be NULL at the time escl_thread_key_delete()
 * is called. It is the responsibility of the application to free any application
 * storage or perform any cleanup actions for data structures related to the deleted
 * key or associated thread-specific data in any threads; this cleanup can be done
 * either before or after pthread_key_delete() is called. Any attempt to use key
 * following the call to pthread_key_delete() results in undefined behavior.
 * The escl_thread_key_delete() function shall be callable from within destructor functions.
 * No destructor functions shall be invoked by escl_thread_key_delete().
 *  Any destructor function that may have been associated with key shall no longer
 * be called upon thread exit.
 *  Parameter     :
 *  -----------
 *  key         :    key obtained via a previous call to escl_thread_key_create().
 *  Return values :
 *  ---------------
 *  If successful, returns zero; otherwise, an error number shall be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_key_delete(key) TlsFree(key)

/************************************************************************
 *
 *         void * escl_thread_getspecific(escl_thread_key_t key)
 *
 *  Description :
 *  -------------
 *  The escl_thread_getspecific() function shall return the value
 *  currently bound to the specified key on behalf of the calling thread.
 *
 *  Parameter     :
 *  -----------
 *  key         :    key obtained via a previous call to escl_thread_key_create().
 *  Return values :
 *  ---------------
 *  the thread-specific data value associated with the given key.
 *  If no thread-specific data value is associated with key, then the
 *  value NULL shall be returned.
 *
 ************************************************************************/
#define escl_thread_getspecific(key) TlsGetValue(key)

/************************************************************************
 *
 *    int escl_thread_getspecific(escl_thread_key_t key, const void *value)
 *
 *  Description :
 *  -------------
 *  The escl_thread_setspecific() function shall associate a thread-specific
 *  value with a key obtained via a previous call to escl_thread_key_create().
 *  Different threads may bind different values to the same key.
 *  These values are typically pointers to blocks of dynamically allocated memory
 *  that have been reserved for use by the calling thread.
 *
 *  Parameter     :
 *  -----------
 *  key         :    key obtained via a previous call to escl_thread_key_create().
 *  value       :    value to be set to thread local storage.
 *  Return values :
 *  ---------------
 *  If successful, returns zero; otherwise, an error number shall be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_setspecific(key, value) ((TlsSetValue(key, value) != 0) ? 0 : -1)

#define escl_mlock(addr, length) (VirtualLock(addr, length) ? 0 : -1)

static inline unsigned escl_getpagesize(void)
{
    SYSTEM_INFO siSysInfo;

    // Copy the hardware information to the SYSTEM_INFO structure.

    GetSystemInfo(&siSysInfo);

    // Display the contents of the SYSTEM_INFO structure.
    return siSysInfo.dwPageSize;
}

#define escl_aligned_malloc(sz, align) _aligned_malloc(sz, align)
#define escl_aligned_free(p) _aligned_free(p)

#define escl_spinlock_init(x) InitializeCriticalSection((x))
#define escl_spinlock_destroy(x) DeleteCriticalSection((x))
#define escl_spinlock_lock(x) EnterCriticalSection((x))
#define escl_spinlock_unlock(x) LeaveCriticalSection((x))
#define escl_spinlock_trylock(x) TryEnterCriticalSection((x))

#define escl_spinlock_lock_irqsave(x, f) escl_spinlock_lock(x)
#define escl_spinlock_unlock_irqrestore(x, f) escl_spinlock_unlock(x)

#define wake_up(w)                         \
    {                                      \
        escl_mutex_lock(&w->lock);         \
        escl_thread_cond_signal(&w->cond); \
        escl_mutex_unlock(&w->lock);       \
    }

#define escl_init_waitqueue_head(w)                           \
    {                                                         \
        assert(escl_mutex_init(&(w)->lock) == 0);             \
        assert(escl_thread_cond_init(&(w)->cond, NULL) == 0); \
    }

/*
#define escl_mutex_init(a)    InitializeCriticalSection((a))
#define escl_mutex_destroy(a) DeleteCriticalSection((a))
#define escl_mutex_lock(a)    EnterCriticalSection((a))
#define escl_mutex_unlock(a) LeaveCriticalSection((a))
#define escl_mutex_trylock(a) TryEnterCriticalSection((a))
 */

/************************************************************************
 *
 *       void escl_semaphore_init(escl_semaphore_t *sem, unsigned val)
 *
 *  Description :
 *  -------------
 *  initialises the unnamed semaphore at the address pointed
 *  to by <sem>. The <val> argument specifies the initial value for the semaphore.
 *
 *  Parameter     :
 *  -----------
 *  sem         :    Pointer to semaphore object.
 *  val         :    Initial value.
 *  Return values :
 *  ---------------
 *  None
 *
 ************************************************************************/

extern void escl_semaphore_init(escl_semaphore_t *sem, unsigned val);

/************************************************************************
 *
 *          void escl_semaphore_destroy(escl_semaphore_t *sem)
 *
 *  Description :
 *  -------------
 *  Destroys the unnamed semaphore at the address pointed to by <sem>.
 *  Only a semaphore that has been initialised by escl_semaphore_init()
 *  should be destroyed using escl_semaphore_destroy().
 *
 *  Parameter     :
 *  -----------
 *  sem           :    Pointer to semaphore object.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/

#define escl_semaphore_destroy(sem) CloseHandle(*(sem))
#define escl_semaphore_post(sem) ReleaseSemaphore(*(sem), 1, NULL)
//#define escl_semaphore_wait(sem) WaitForSingleObject(sem, INFINITE)
#define escl_semaphore_trywait(sem) ((WaitForSingleObject(*(sem), 0) == WAIT_OBJECT_0))
extern int escl_semaphore_timed_wait(escl_semaphore_t *sem, unsigned msecs);
extern int escl_semaphore_wait(escl_semaphore_t *sem);

extern int escl_thread_cond_init(escl_cond_t *cv, escl_mutex_t *mutex);
extern void escl_thread_cond_destroy(escl_cond_t *cv);
extern int escl_thread_cond_timed_wait(escl_cond_t *cv, escl_mutex_t *mutex, unsigned msecs);
extern int escl_thread_cond_wait(escl_cond_t *cv, escl_mutex_t *mutex);
extern int escl_thread_cond_signal(escl_cond_t *cv);
extern int escl_thread_cond_broadcast(escl_cond_t *cv);

#if defined(_MSC_VER) && (_MSC_VER >= 1400)

#else
#define __FUNCTION__ "Unknown"
#endif

#if 0  //! defined(__FUNCTION__)

#pragma comment(lib, "imagehlp.lib")
#pragma warning(disable : 4172)  // returning address of a temp

#define BUFFERSIZE 512

BOOL    InitSymInfo(PCSTR lpszInitialSymbolPath, BOOL bSysPath);
void    InitSymPath(PSTR lpszSymbolPath, PCSTR lpszIniPath, BOOL bSysPath);
BOOL    GetFuncInfo(ULONG fnAddress, ULONG stackAddress, LPTSTR lpszSymbol);
LPCTSTR GetFuncName();
BOOL    KillSymInfo();

#define __FUNCTION__ GetFuncName()
#define __func__ __FUNCTION__

#endif  //__FUNCTION__

#elif defined(__ECOS__)

/**
 * @def tmp
 *
 */
/**
 * @def  ESCL_THREAD_PRIORITY_LOWEST
 * @brief Priority class of a thread.
 */
#define ESCL_THREAD_PRIORITY_LOWEST (32)

/**
 * @def ESCL_THREAD_PRIORITY_BELOW_NORMAL
 * @brief Define the below normal priority of a thread
 */
#define ESCL_THREAD_PRIORITY_BELOW_NORMAL (24)

/**
 * @def ESCL_THREAD_PRIORITY_NORMAL
 * @brief Normal priority for the priority class.
 */
#define ESCL_THREAD_PRIORITY_NORMAL (16)

/**
 * @def ESCL_THREAD_PRIORITY_ABOVE_NORMAL
 * @brief Above normal priority for the priority class.
 */
#define ESCL_THREAD_PRIORITY_ABOVE_NORMAL (8)

/**
 * @def ESCL_THREAD_PRIORITY_CRITICAL
 * @brief Define the critical priority for the priority class.
 */
#define ESCL_THREAD_PRIORITY_CRITICAL (4)

/**
 * @def ESCL_THREAD_PRIORITY_HIGHEST
 * @brief Define the highest priority of thread.
 */
#define ESCL_THREAD_PRIORITY_HIGHEST (1)

/**
 * @def ESCL_THREAD_STACK_SIZE
 * @brief Normally stack size of a thread in ECOS.
 */
#define ESCL_THREAD_STACK_SIZE (2048)

/**
 * @typedef escl_spinlock_t
 * @brief Type definition of spin lock(eCos).
 */
typedef cyg_spinlock_t escl_spinlock_t;

/**
 * @typedef escl_mutex_t
 * @brief Type definition of mutex lock(eCos).
 */
typedef cyg_mutex_t escl_mutex_t;

/**
 * @typedef escl_thread_ret_t
 * @brief Type definition of thread's return value(eCos).
 */
typedef void *escl_thread_ret_t;

/**
 * @typedef escl_thread_ret_t
 * @brief Type definition of thread's argument(WIN32).
 */
typedef void *escl_thread_data_t;

/**
 * @typedef escl_cond_t
 * @brief Type definition of condition variable.
 */
typedef cyg_cond_t escl_cond_t;
// Type definition of timer routine.
// typedef void (* escl_timer_routine_t)(void *arg);
// Type definition of thread handler. void (*func)(unsigned long)
typedef escl_thread_ret_t(__calltype *escl_thread_func_t)(escl_thread_data_t arg);

/**
 * @typedef escl_semaphore_t
 * @brief Type definition of semaphore variable.
 */
typedef cyg_sem_t escl_semaphore_t;

/**
 * @struct _ESCL_THREAD_T
 * @brief Type definition of thread handle.
 */
typedef struct _ESCL_THREAD_T {
    cyg_handle_t handle;      ///< Thread id
    cyg_thread object;        ///< Thread object, eCos needs that.
    escl_thread_data_t arg;   ///< arguments
    escl_thread_func_t func;  ///< routine.
    uint32_t *stack;          ///< Pointer to Thread stack.
} escl_thread_t;
typedef escl_thread_t ESCL_THREAD_T;

#if defined(CYGPKG_IO)
typedef cyg_io_handle_t escl_io_handle_t;
#endif

#define escl_io_open cyg_io_lookup
#define escl_io_close cyg_io_close

/************************************************************************
 *
 *                      int escl_thread_create
 *                         (
 *                          escl_thread_t        *tid,
 *                          escl_thread_func_t    func,
 *                          escl_thread_data_t    arg,
 *                          unsigned             stack_size
 *                          )
 *  Description :
 *  -------------
 *   The escl_thread_create() function is used to create a new thread, with
 *   stack size specified by stack_size(The default size was defined by
 *   ESCL_THREAD_STACK_SIZE), Upon successful completion, escl_thread_create()
 *   will store the ID of the created thread in the location specified by tid.
 *
 *   Upon its creation, the thread executes func, with arg as its
 *   sole argument.  If start_routine returns, the effect is as if there was
 *   an implicit call return instruction,
 *
 *  Parameter :
 *  -----------
 *  tid        :    returned handle to the thread,
 *                  This pointer indicates where the new thread id should be put into.
 *  func       :    Routine that begins execution of the thread.
 *  arg        :    data value passed to the thread entry routine.
 *  stack_size :    Size, In bytes, of the stack for the thread.
 *  ---------------
 *  If successful, the escl_thread_create() function will return zero.
 *  Otherwise, an error number will be returned to indicate the error.
 *
 ************************************************************************/

#ifdef __ESCL_DEBUG__
#define __ESCL_DBG_STACK_STAT__ /* the option shouldn't be enabled when __ESCL_DEBUG__ is disabled.                   \
                                 * If you are willing to do that, increasing the size of the stack pool in            \
                                 * escl_config.h, to avoid the stack pool be exhausted by the space margine for stack \
                                 * size probing.                                                                      \
                                 */
#endif                          //__ESCL_DEBUG__

#ifdef __ESCL_DBG_STACK_STAT__
extern int escl_thread_create_debug(char *called_by, escl_thread_t *tid, escl_thread_func_t func,
                                    escl_thread_data_t arg, unsigned stack_size);

#define escl_thread_create(tid, func, arg, stack_size) \
    escl_thread_create_debug(__FUNCTION__, tid, func, arg, stack_size)

#else
extern int escl_thread_create(escl_thread_t *tid, escl_thread_func_t func, escl_thread_data_t arg, unsigned stack_size);
#endif

/************************************************************************
 *
 *                  escl_thread_t escl_thread_self(void)
 *  Description :
 *  -------------
 *   The escl_thread_self() function shall return the thread ID of
 *   the calling thread.
 *
 *  Parameter :
 *  -----------
 *   N/A
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_self() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
#define escl_thread_self() cyg_thread_self()  // TODO. should return escl_thread_t.

/************************************************************************
 *
 *                  int escl_thread_cancel(escl_thread_t tid)
 *  Description :
 *  -------------
 *  escl_thread_cancel() is used to cause a thread to exit.
 *  In most system will frees the thread's resources(such as stack)
 *  except eCos. you should call escl_thread_join() to free the allocated
 *  resources of the thread.
 *
 *
 *  Parameter :
 *  -----------
 *   tid      :  Thread ID
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_cancel() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
#define escl_thread_cancel(tid) ((cyg_thread_delete((tid).handle)) ? 0 : ERR_THREAD)

/************************************************************************
 *
 *                  void escl_thread_exit(unsigned long ret)
 *  Description :
 *  -------------
 *  terminate the calling thread,
 *
 *
 *  Parameter :
 *  -----------
 *   ret      return value of called thread.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_exit(ret) cyg_thread_exit()
/************************************************************************
 *
 *             void escl_thread_destroy(escl_thread_t tid)
 *  Description :
 *  -------------
 *   Free the allocated stack of a thread. it is necessary for eCos.
 *
 *  Parameter :
 *  -----------
 *   tid      Thread ID.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_destroy(tid)                                             \
    ({                                                                       \
        ESCL_ASSERT_FREE((tid).stack); /* cyg_mutex_destroy(&(tid.mutex));*/ \
    })

/************************************************************************
 *
 *      int escl_thread_setpriority(escl_thread_t tid, const int priority)
 *  Description :
 *  -------------
 *  Set the thread to the specified priority level. The valid ranges for the priority value are
 *  determined by the ESCL_THREAD_PRIORITY_LOWEST and ESCL_THREAD_PRIORITY_HIGHEST.
 *  default priority is determined by ESCL_THREAD_PRIORITY_NORMAL.
 *  eCos  from 1 to 32, where lower values represent higher thread priorities.
 *  Win32 from THREAD_PRIORITY_LOWEST to THREAD_PRIORITY_HIGHEST
 *  linux from 0 to 99.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  priority    :       priority of thread will be set.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/

#define escl_thread_setpriority(tid, priority)             \
    ({                                                     \
        cyg_thread_set_priority((tid).handle, (priority)); \
        (0);                                               \
    })

/************************************************************************
 *
 *      int escl_thread_getpriority(escl_thread_t tid)
 *
 *  Description :
 *  -------------
 *  Retrieves the priority value for the specified thread.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  The return value is the thread's priority level.
 ************************************************************************/
#define escl_thread_getpriority(tid) cyg_thread_get_priority((tid).handle)

/************************************************************************
 *
 *                 void escl_thread_suspend(escl_thread_t tid)
 *  Description :
 *  -------------
 *  Postpones the execution of a thread. If a thread is suspended multiple times, the resume
 *  function call must be made for each suspend function call before the thread will execute.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_suspend(tid) cyg_thread_suspend((tid).handle)

/************************************************************************
 *
 *               void escl_thread_resume(escl_thread_t tid)
 *  Description :
 *  -------------
 *  Causes a thread to continue execution. This call must be made after a thread is created in
 *  order to start the execution of the thread. Each suspend function call must correspond to a
 *  resume function call before a thread will continue execution.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_resume(tid) cyg_thread_resume((tid).handle)

/************************************************************************
 *
 *                void escl_thread_delay(unsigned msecs)
 *
 *  Description :
 *  -------------
 *  Suspends execution of the calling thread for (at least) msecs microseconds.
 *  The sleep may be lengthened slightly by any system activity or by the time
 *  spent processing the call or by the granularity of system timers.
 *
 *  Parameter :
 *  -----------
 *  msecs         :          time for thread to sleep (in microseconds).
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
extern void escl_thread_delay(unsigned msecs);

/************************************************************************
 *
 *                 void escl_thread_join(escl_thread_t tid)
 *  Description :
 *  -------------
 *  function suspends execution of the calling thread
 *  until the target thread terminates, unless the target thread has already
 *  terminated.
 *  This does release resources allocated for the thread.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/

extern int __IMP_escl_thread_join(escl_thread_t *tid);
#define escl_thread_join(tid) __IMP_escl_thread_join(&(tid))
/************************************************************************
 *
 *         int escl_mutex_init(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  Creates or opens a named or unnamed mutex object.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_init(a) cyg_mutex_init(a)

/************************************************************************
 *
 *         int escl_mutex_init(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  Creates or opens a named or unnamed mutex object.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_destroy(a) cyg_mutex_destroy(a)

/************************************************************************
 *
 *         int escl_mutex_lock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The mutex object referenced by mutex shall be locked by calling escl_mutex_lock().
 *  If the mutex is already locked,
 *  the calling thread shall block until the mutex becomes available.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_lock(a) cyg_mutex_lock(a)
/************************************************************************
 *
 *         int escl_mutex_unlock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_unlock() function shall release the mutex object
 *  referenced by mutex. The manner in which a mutex is released is
 *  dependent upon the OS. If there are threads
 *  blocked on the mutex object referenced by mutex when escl_mutex_unlock() is called,
 *  resulting in the mutex becoming available,
 *  the scheduling policy shall determine which thread shall acquire the mutex.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_unlock(a) cyg_mutex_unlock((a))

/************************************************************************
 *
 *         int escl_mutex_trylock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_trylock() function locks mutex.  If the mutex is
 *  already locked, escl_mutex_trylock() will not block waiting for the
 *   mutex, but will return an error condition.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  "true" if the mutex was locked, "false" if the mutex couldn't be locked.
 *
 ************************************************************************/
#define escl_mutex_trylock(a) (cyg_mutex_trylock((a)) == TRUE)

/************************************************************************
 *
 *       void escl_semaphore_init(escl_semaphore_t *sem, unsigned val)
 *
 *  Description :
 *  -------------
 *  initialises the unnamed semaphore at the address pointed
 *  to by <sem>. The <val> argument specifies the initial value for the semaphore.
 *
 *  Parameter     :
 *  -----------
 *  sem         :    Pointer to semaphore object.
 *  val         :    Initial value.
 *  Return values :
 *  ---------------
 *  None
 *
 ************************************************************************/

#define escl_semaphore_init(sem, val) cyg_semaphore_init(sem, val)

/************************************************************************
 *
 *          void escl_semaphore_destroy(escl_semaphore_t *sem)
 *
 *  Description :
 *  -------------
 *  Destroys the unnamed semaphore at the address pointed to by <sem>.
 *  Only a semaphore that has been initialised by escl_semaphore_init()
 *  should be destroyed using escl_semaphore_destroy().
 *
 *  Parameter     :
 *  -----------
 *  sem           :    Pointer to semaphore object.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_semaphore_destroy(sem) cyg_semaphore_destroy(sem)
// ncrements (unlocks) the semaphore pointed to by sem. If the semaphore's value consequently becomes greater than zero,
// then another process or thread blocked in a escl_semaphore_wait(3) call will be woken up and proceed to lock the
// semaphore.
#define escl_semaphore_post(sem) cyg_semaphore_post(sem)
#define escl_semaphore_wait(sem) cyg_semaphore_wait(sem)
#define escl_semaphore_trywait(sem) cyg_semaphore_trywait(sem)
#define escl_semaphore_getvalue(sem, rval) cyg_semaphore_peek(sem, rval)
#define escl_semaphore_timed_wait(sem, msecs) \
    cyg_semaphore_timed_wait(sem, (cyg_tick_count_t)(cyg_current_time() + ((msecs) / 10)))

/************************************************************************
 *
 *                   void escl_thread_yield(void)
 *  Description :
 *  -------------
 *   The escl_thread_yield() function resets the accumulated time slice of the
 *   current thread and then enters the thread scheduler, which resumes execu-
 *   tion of the next thread ready to run.  If no other thread is ready to
 *   run, control returns to the current thread.
 *
 *  Parameter :
 *  -----------
 *   N/A
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_yield() cyg_thread_yield()  // escl_thread_delay(0)   //cyg_thread_yield()

extern int escl_thread_cancel_wait(escl_thread_t tid);

#define escl_spinlock_init(x) cyg_spinlock_init((x), false)
#define escl_spinlock_destroy(x) cyg_spinlock_destroy((x))
#define escl_spinlock_lock(x) cyg_spinlock_spin((x))
#define escl_spinlock_unlock(x) cyg_spinlock_clear((x))
#define escl_spinlock_trylock(x) cyg_spinlock_try((x))
#define escl_spinlock_lock_irqsave(x, flags) cyg_spinlock_spin_intsave((x), (flags))
#define escl_spinlock_unlock_irqrestore(x, flags) cyg_spinlock_clear_intsave((x), (flags))

#define escl_spinlock_lock_irqsave(x, f) escl_spinlock_lock(x)
#define escl_spinlock_unlock_irqrestore(x, f) escl_spinlock_unlock(x)

#define wake_up(w)                         \
    ({                                     \
        escl_mutex_lock(&w->lock);         \
        escl_thread_cond_signal(&w->cond); \
        escl_mutex_unlock(&w->lock);       \
    })

#define escl_init_waitqueue_head(w)                    \
    ({                                                 \
        escl_mutex_init(&(w)->lock);                   \
        escl_thread_cond_init(&(w)->cond, &(w)->lock); \
    })

extern int escl_thread_cond_init(escl_cond_t *cv, escl_mutex_t *mutex);
extern void escl_thread_cond_destroy(escl_cond_t *cv);
extern int escl_thread_cond_timed_wait(escl_cond_t *cv, escl_mutex_t *mutex, unsigned msecs);
extern int escl_thread_cond_wait(escl_cond_t *cv, escl_mutex_t *mutex);
extern int escl_thread_cond_signal(escl_cond_t *cv);
extern int escl_thread_cond_broadcast(escl_cond_t *cv);

typedef cyg_flag_t escl_flag_t;
typedef cyg_flag_value_t escl_flag_value_t;
typedef cyg_flag_mode_t escl_flag_mode_t;

#define ESCL_FLAG_WAITMODE_AND CYG_FLAG_WAITMODE_AND
#define ESCL_FLAG_WAITMODE_OR CYG_FLAG_WAITMODE_OR
#define ESCL_FLAG_WAITMODE_CLR CYG_FLAG_WAITMODE_CLR

#define escl_flag_init(pflag) cyg_flag_init(pflag)
#define escl_flag_destroy(pflag) cyg_flag_destroy(pflag)
#define escl_flag_setbits(pflag, value) cyg_flag_setbits(pflag, value)
#define escl_flag_maskbits(pflag, value) cyg_flag_maskbits(pflag, value)
#define escl_flag_wait(pflag, pattern, mode) cyg_flag_wait(pflag, pattern, mode)
#define escl_flag_timed_wait(pflag, pattern, mode, msecs) \
    cyg_flag_timed_wait(pflag, pattern, mode, (cyg_tick_count_t)(cyg_current_time() + ((msecs) / 10)))
#define escl_flag_poll(pflag, pattern, mode) cyg_flag_poll(pflag, pattern, mode)
#define escl_flag_peek(pflag) cyg_flag_peek(pflag)
#define escl_flag_waiting(pflag) cyg_flag_waiting(pflag)

extern void cyg_flag_init(cyg_flag_t *flag);
extern void cyg_flag_destroy(cyg_flag_t *flag);
extern void cyg_flag_setbits(cyg_flag_t *flag, cyg_flag_value_t value);
extern void cyg_flag_maskbits(cyg_flag_t *flag, cyg_flag_value_t value);

extern cyg_flag_value_t cyg_flag_wait(cyg_flag_t *flag, cyg_flag_value_t pattern, cyg_flag_mode_t mode);
extern cyg_flag_value_t cyg_flag_timed_wait(cyg_flag_t *flag, cyg_flag_value_t pattern, cyg_flag_mode_t mode,
                                            cyg_tick_count_t abstime);
extern cyg_flag_value_t cyg_flag_poll(cyg_flag_t *flag, cyg_flag_value_t pattern, cyg_flag_mode_t mode);
extern cyg_flag_value_t cyg_flag_peek(cyg_flag_t *flag);
extern cyg_bool_t cyg_flag_waiting(cyg_flag_t *flag);

#elif defined(LINUX) || defined(__linux) || defined(__CYGWIN__)
// #include <pthread.h>

#if defined(__KERNEL__)
// #include <asm/semaphore.h>
// #include <linux/spinlock.h>

// Priority class of a thread. That isn't supported by kernel mode.
#define ESCL_THREAD_PRIORITY_LOWEST (0)
#define ESCL_THREAD_PRIORITY_BELOW_NORMAL (0)
// Normal priority for the priority class.
#define ESCL_THREAD_PRIORITY_NORMAL (0)
#define ESCL_THREAD_PRIORITY_ABOVE_NORMAL (0)
#define ESCL_THREAD_PRIORITY_CRITICAL (0)
// The highest priority of thread.
#define ESCL_THREAD_PRIORITY_HIGHEST (0)

// Normally stack size of a thread in kernel.
#define ESCL_THREAD_STACK_SIZE (KERNEL_STACK_SIZE)

// Type definition of spin lock(Linux Kernel).
typedef spinlock_t escl_spinlock_t;
// Type definition of thread's handle.
typedef struct __ESCL_THREAD_T {
    pid_t pid;
    int ret;
    struct task_struct *handle;
    escl_thread_func_t func;
    escl_thread_data_t arg;
    struct completion start;
    struct completion exit;
} escl_thread_t;

// Type definition of thread's return value(Linux Kernel).
typedef int escl_thread_ret_t;
// Type deifnition of thread's argument(Linux Kernel)
typedef void *escl_thread_data_t;
// Type definition of semaphore.
typedef struct semaphore escl_semaphore_t;

// Type definition of thread handler.
typedef escl_thread_ret_t(__calltype *escl_thread_func_t)(escl_thread_data_t arg);

/************************************************************************
 *
 *                      int escl_thread_create
 *                         (
 *                          escl_thread_t        *tid,
 *                          escl_thread_func_t    func,
 *                          escl_thread_data_t    arg,
 *                          unsigned             stack_size
 *                          )
 *  Description :
 *  -------------
 *   The escl_thread_create() function is used to create a new thread, with
 *   stack size specified by stack_size(The default size was defined by
 *   ESCL_THREAD_STACK_SIZE), Upon successful completion, escl_thread_create()
 *   will store the ID of the created thread in the location specified by tid.
 *
 *   Upon its creation, the thread executes func, with arg as its
 *   sole argument.  If start_routine returns, the effect is as if there was
 *   an implicit call return instruction,
 *
 *  Parameter :
 *  -----------
 *  tid        :    returned handle to the thread,
 *                  This pointer indicates where the new thread id should be put into.
 *  func       :    Routine that begins execution of the thread.
 *  arg        :    data value passed to the thread entry routine.
 *  stack_size :    Size, In bytes, of the stack for the thread.
 *  ---------------
 *  If successful, the escl_thread_create() function will return zero.
 *  Otherwise, an error number will be returned to indicate the error.
 *
 ************************************************************************/
extern int escl_thread_create(escl_thread_t *tid, escl_thread_func_t func, escl_thread_data_t arg, unsigned stack_size);

/*
#define escl_thread_create(tid, func, arg, stack_size)
(*(tid) = kernel_thread((func), (arg)  CLONE_FS | CLONE_FILES | CLONE_SIGHAND))


#define kthread_create(tid, func, arg, stack_size)
 */

/************************************************************************
 *
 *                  int escl_thread_cancel(escl_thread_t tid)
 *  Description :
 *  -------------
 *  escl_thread_cancel() is used to cause a thread to exit.
 *  In most system will frees the thread's resources(such as stack)
 *  except eCos. you should call escl_thread_join() to free the allocated
 *  resources of the thread.
 *
 *
 *  Parameter :
 *  -----------
 *   tid      :  Thread ID
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_cancel() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
extern int __IMP_escl_thread_cancel(escl_thread_t *tid);
#define escl_thread_cancel(tid) __IMP_escl_thread_cancel(&(tid))

/************************************************************************
 *
 *              int escl_thread_join(escl_thread_t tid);
 *  Description :
 *  -------------
 *  The escl_thread_join() function suspends execution of the calling thread
 *  until the target thread terminates, unless the target thread has already
 *��terminated.
 *
 *  Parameter :
 *  -----------
 *   tid      :   Target thread ID.
 *
 *  Return values:
 *  ---------------
 *  If successful,  the escl_thread_join() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
extern void __IMP_escl_thread_join(escl_thread_t *tid);
#define escl_thread_join(tid) __IMP_escl_thread_join(&(tid))

/************************************************************************
 *
 *                  void escl_thread_exit(unsigned long ret)
 *  Description :
 *  -------------
 *  terminate the calling thread,
 *
 *
 *  Parameter :
 *  -----------
 *   ret      return value of called thread.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_exit() exit_thread()

/************************************************************************
 *
 *             void escl_thread_destroy(escl_thread_t tid)
 *  Description :
 *  -------------
 *   Free the allocated stack of a thread. it is necessary for eCos.
 *
 *  Parameter :
 *  -----------
 *   tid      Thread ID.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_destroy(tid) __ESCL_NOP__

/************************************************************************
 *
 *      int escl_thread_getpriority(escl_thread_t tid)
 *
 *  Description :
 *  -------------
 *  Retrieves the priority value for the specified thread.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  The return value is the thread's priority level.
 ************************************************************************/
#define escl_thread_getpriority(tid) (0)

/************************************************************************
 *
 *      int escl_thread_setpriority(escl_thread_t tid, const int priority)
 *
 *  Description :
 *  -------------
 *  Set the thread to the specified priority level. The valid ranges for the priority value are
 *  determined by the ESCL_THREAD_PRIORITY_LOWEST and ESCL_THREAD_PRIORITY_HIGHEST.
 *  default priority is determined by ESCL_THREAD_PRIORITY_NORMAL.
 *  eCos  from 1 to 32, where lower values represent higher thread priorities.
 *  Win32 from THREAD_PRIORITY_LOWEST to THREAD_PRIORITY_HIGHEST
 *  linux from 0 to 99.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  priority    :       priority of thread will be set.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_thread_setpriority() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_setpriority(tid, priority) (0)

/************************************************************************
 *
 *                void escl_thread_delay(unsigned msecs)
 *
 *  Description :
 *  -------------
 *  Suspends execution of the calling thread for (at least) msecs microseconds.
 *  The sleep may be lengthened slightly by any system activity or by the time
 *  spent processing the call or by the granularity of system timers.
 *
 *  Parameter :
 *  -----------
 *  msecs         :          time for thread to sleep (in microseconds).
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_delay(msecs) msleep(msecs)  // schedue_timeout(xxx,) //mdelay(msecs)

/************************************************************************
 *
 *       void escl_semaphore_init(escl_semaphore_t *sem, unsigned val)
 *
 *  Description :
 *  -------------
 *  initialises the unnamed semaphore at the address pointed
 *  to by <sem>. The <val> argument specifies the initial value for the semaphore.
 *
 *  Parameter     :
 *  -----------
 *  sem         :    Pointer to semaphore object.
 *  val         :    Initial value.
 *  Return values :
 *  ---------------
 *  None
 *
 ************************************************************************/

#define escl_semaphore_init(sem, val) sema_init(sem, val)

/************************************************************************
 *
 *          void escl_semaphore_destroy(escl_semaphore_t *sem)
 *
 *  Description :
 *  -------------
 *  Destroys the unnamed semaphore at the address pointed to by <sem>.
 *  Only a semaphore that has been initialised by escl_semaphore_init()
 *  should be destroyed using escl_semaphore_destroy().
 *
 *  Parameter     :
 *  -----------
 *  sem           :    Pointer to semaphore object.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/

#define escl_semaphore_destroy(sem) __ESCL_NOP__
#define escl_semaphore_wait(sem) down(sem)
#define escl_semaphore_post(sem) up(sem)
#define escl_semaphore_trywait(sem) ((down_trylock(sem) == 0))
#define escl_semaphore_getvalue(sem) (sem->count)
extern int escl_semaphore_timed_wait(escl_semaphore_t *sem, unsigned msecs);

#define escl_spinlock_init(x) spin_lock_init((x))
#define escl_spinlock_destroy(x) __ESCL_NOP__

#define escl_spinlock_lock(x) spin_lock((x))
#define escl_spinlock_unlock(x) spin_unlock((x))
#define escl_spinlock_trylock(x) spin_trylock((x))
#define escl_spinlock_lock_irqsave(x, flags) spin_lock_irqsave((x), (flags))
#define escl_spinlock_unlock_irqrestore(x, flags) spin_unlock_irqrestore((x), (flags))

#else  //!__KERNEL__
// #include <sys/time.h>
// #include <unistd.h>

// #define __USE_XOPEN2K
// #include <semaphore.h>

#define myGetPid() getpid()
#define vmalloc(sz) malloc(sz)
#define vfree(p) free(p)
#ifndef GFP_KERNEL
#define GFP_KERNEL 0
#endif /*GFP_KERNEL*/
#define kmalloc(sz, flag) malloc(sz)
#define kfree(p) free(p)

// Priority class of a thread. That isn't supported by kernel mode.
#define ESCL_THREAD_PRIORITY_LOWEST sched_get_priority_min(SCHED_OTHER)
// The highest priority of thread.
#define ESCL_THREAD_PRIORITY_HIGHEST sched_get_priority_max(SCHED_OTHER)
// Normal priority for the priority class.
#define ESCL_THREAD_PRIORITY_NORMAL (((ESCL_THREAD_PRIORITY_LOWEST) + (ESCL_THREAD_PRIORITY_HIGHEST)) / 2)
#define ESCL_THREAD_PRIORITY_BELOW_NORMAL ((ESCL_THREAD_PRIORITY_NORMAL)-1)
#define ESCL_THREAD_PRIORITY_ABOVE_NORMAL ((ESCL_THREAD_PRIORITY_NORMAL) + 1)
#define ESCL_THREAD_PRIORITY_CRITICAL ((ESCL_THREAD_PRIORITY_ABOVE_NORMAL) + 1)

// Normally stack size of a thread in POSIX.
#define ESCL_THREAD_STACK_SIZE (1024 * 1024)
// Type definition of thread handle.
typedef pthread_t escl_thread_t;
// Type definition of mutex lock(POSIX).
typedef pthread_mutex_t escl_mutex_t;
// Type definition of thread's return value(POSIX).
typedef void *escl_thread_ret_t;
// Type deifnition of thread's argument(POSIX)
typedef void *escl_thread_data_t;
// Default stack size of POSIX thread.
#define ESCL_THREAD_STACK_SIZE (1024 * 1024)
// Type definition of semaphore.
typedef sem_t escl_semaphore_t;
// Type definition of condition variable.
typedef pthread_cond_t escl_cond_t;
// Type definition of timer routine.
// typedef void (* escl_timer_routine_t)(void *arg);
// Type definition of thread handler.
typedef escl_thread_ret_t (*escl_thread_func_t)(escl_thread_data_t arg);

typedef pthread_key_t escl_thread_key_t;

/************************************************************************
 *
 *                      int escl_thread_create
 *                         (
 *                          escl_thread_t        *tid,
 *                          escl_thread_func_t    func,
 *                          escl_thread_data_t    arg,
 *                          unsigned             stack_size
 *                          )
 *  Description :
 *  -------------
 *   The escl_thread_create() function is used to create a new thread, with
 *   stack size specified by stack_size(The default size was defined by
 *   ESCL_THREAD_STACK_SIZE), Upon successful completion, escl_thread_create()
 *   will store the ID of the created thread in the location specified by tid.
 *
 *   Upon its creation, the thread executes func, with arg as its
 *   sole argument.  If start_routine returns, the effect is as if there was
 *   an implicit call return instruction,
 *
 *  Parameter :
 *  -----------
 *  tid        :    returned handle to the thread,
 *                  This pointer indicates where the new thread id should be put into.
 *  func       :    Routine that begins execution of the thread.
 *  arg        :    data value passed to the thread entry routine.
 *  stack_size :    Size, In bytes, of the stack for the thread.
 *  ---------------
 *  If successful, the escl_thread_create() function will return zero.
 *  Otherwise, an error number will be returned to indicate the error.
 *
 ************************************************************************/
extern int escl_thread_create(escl_thread_t *tid, escl_thread_func_t func, escl_thread_data_t arg, unsigned stack_size);

/************************************************************************
 *
 *                  escl_thread_t escl_thread_self(void)
 *  Description :
 *  -------------
 *   The escl_thread_self() function shall return the thread ID of
 *   the calling thread.
 *
 *  Parameter :
 *  -----------
 *   N/A
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_self() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
#define escl_thread_self() pthread_self()

/************************************************************************
 *
 *              int escl_thread_join(escl_thread_t tid);
 *  Description :
 *  -------------
 *  The escl_thread_join() function suspends execution of the calling thread
 *  until the target thread terminates, unless the target thread has already
 *��terminated.
 *
 *  Parameter :
 *  -----------
 *   tid      :   Target thread ID.
 *
 *  Return values:
 *  ---------------
 *  If successful,  the escl_thread_join() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_join(a) pthread_join((a), NULL)

/************************************************************************
 *
 *                  int escl_thread_cancel(escl_thread_t tid)
 *  Description :
 *  -------------
 *  escl_thread_cancel() is used to cause a thread to exit.
 *  In most system will frees the thread's resources(such as stack)
 *  except eCos. you should call escl_thread_join() to free the allocated
 *  resources of the thread.
 *
 *
 *  Parameter :
 *  -----------
 *   tid      :  Thread ID
 *
 *  Return values:
 *  ---------------
 *  The escl_thread_cancel() function returns the thread ID of the
 *  calling thread.
 *
 ************************************************************************/
#define escl_thread_cancel(tid) pthread_cancel(tid)

/************************************************************************
 *
 *                  void escl_thread_exit(unsigned long ret)
 *  Description :
 *  -------------
 *  terminate the calling thread.
 *
 *
 *  Parameter :
 *  -----------
 *   ret      return value of called thread.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_exit(ret) pthread_exit((void *)(ret))

/************************************************************************
 *
 *             void escl_thread_destroy(escl_thread_t tid)
 *  Description :
 *  -------------
 *   Free the allocated stack of a thread. it is necessary for eCos.
 *
 *  Parameter :
 *  -----------
 *   tid      Thread ID.
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/
#define escl_thread_destroy(tid) __ESCL_NOP__

/************************************************************************
 *
 *      int escl_thread_getpriority(escl_thread_t tid)
 *
 *  Description :
 *  -------------
 *  Retrieves the priority value for the specified thread.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  The return value is the thread's priority level.
 ************************************************************************/
extern int escl_thread_getpriority(escl_thread_t tid);

/************************************************************************
 *
 *      int escl_thread_setpriority(escl_thread_t tid, const int priority)
 *
 *  Description :
 *  -------------
 *  Set the thread to the specified priority level. The valid ranges for the priority value are
 *  determined by the ESCL_THREAD_PRIORITY_LOWEST and ESCL_THREAD_PRIORITY_HIGHEST.
 *  default priority is determined by ESCL_THREAD_PRIORITY_NORMAL.
 *  eCos  from 1 to 32, where lower values represent higher thread priorities.
 *  Win32 from THREAD_PRIORITY_LOWEST to THREAD_PRIORITY_HIGHEST
 *  linux from 0 to 99.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  priority    :       priority of thread will be set.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_thread_setpriority() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
extern int escl_thread_setpriority(escl_thread_t tid, int priority);

/************************************************************************
 *
 *         int escl_mutex_init(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  Creates or opens a named or unnamed mutex object.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_init(a) pthread_mutex_init((a), NULL)
/************************************************************************
 *
 *         int escl_mutex_init(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  Creates or opens a named or unnamed mutex object.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_destroy(a) pthread_mutex_destroy((a))
/************************************************************************
 *
 *         int escl_mutex_lock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The mutex object referenced by mutex shall be locked by calling escl_mutex_lock().
 *  If the mutex is already locked,
 *  the calling thread shall block until the mutex becomes available.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_lock(a) pthread_mutex_lock((a))

/************************************************************************
 *
 *         int escl_mutex_unlock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_unlock() function shall release the mutex object
 *  referenced by mutex. The manner in which a mutex is released is
 *  dependent upon the OS. If there are threads
 *  blocked on the mutex object referenced by mutex when escl_mutex_unlock() is called,
 *  resulting in the mutex becoming available,
 *  the scheduling policy shall determine which thread shall acquire the mutex.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  If successful,  the escl_mutex_init() function will return zero.  Otherwise,
 *  an error number will be returned to indicate the error.
 *
 ************************************************************************/
#define escl_mutex_unlock(a) pthread_mutex_unlock((a))

/************************************************************************
 *
 *         int escl_mutex_trylock(escl_mutex_t *mutex)
 *
 *  Description :
 *  -------------
 *  The escl_mutex_trylock() function locks mutex.  If the mutex is
 *  already locked, escl_mutex_trylock() will not block waiting for the
 *   mutex, but will return an error condition.
 *
 *  Parameter     :
 *  -----------
 *  mutex         :    Pointer to mutex object.
 *  Return values :
 *  ---------------
 *  "true" if the mutex was locked, "false" if the mutex couldn't be locked.
 *
 ************************************************************************/
#define escl_mutex_trylock(a) (pthread_mutex_trylock((a)) == 0)

/************************************************************************
 *
 *   int escl_thread_key_create(escl_thread_key_t *key, void (*destructor)(void*));
 *
 *  Description :
 *  -------------
 *  The escl_thread_key_create() function shall create a thread-specific data key
 *  visible to all threads in the process. Key values provided by escl_thread_key_create()
 *  are opaque objects used to locate thread-specific data. Although the same key
 *  value may be used by different threads, the values bound to the key by
 *  escl_thread_setspecific() are maintained on a per-thread basis and persist
 *  for the life of the calling thread.
 *  Parameter     :
 *  -----------
 *  key         :    pointer to received the created key.
 *  func        :    An optional destructor function may be associated with each
 *                   key value. At thread exit, if a key value has a non-NULL destructor pointer,
 *                   and the thread has a non-NULL value associated with that key, the value of the
 *                   key is set to NULL, and then the function pointed to is called with the previously
 *                   associated value as its sole argument. The order of destructor calls is unspecified
 *                   if more than one destructor exists for a thread when it exits
 *  Return values :
 *  ---------------
 *  If successful, returns zero; otherwise, an error number shall be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_key_create(key, func) pthread_key_create(key, func)

/************************************************************************
 *
 *   int escl_thread_key_delete(escl_thread_key_t key);
 *
 *  Description :
 *  -------------
 *  The escl_thread_key_delete() function shall delete a thread-specific data
 *  key previously returned by escl_thread_key_create(). The thread-specific data
 * values associated with key need not be NULL at the time escl_thread_key_delete()
 * is called. It is the responsibility of the application to free any application
 * storage or perform any cleanup actions for data structures related to the deleted
 * key or associated thread-specific data in any threads; this cleanup can be done
 * either before or after pthread_key_delete() is called. Any attempt to use key
 * following the call to pthread_key_delete() results in undefined behavior.
 * The escl_thread_key_delete() function shall be callable from within destructor functions.
 * No destructor functions shall be invoked by escl_thread_key_delete().
 *  Any destructor function that may have been associated with key shall no longer
 * be called upon thread exit.
 *  Parameter     :
 *  -----------
 *  key         :    key obtained via a previous call to escl_thread_key_create().
 *  Return values :
 *  ---------------
 *  If successful, returns zero; otherwise, an error number shall be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_key_delete(key) pthread_key_delete(key)

/************************************************************************
 *
 *         void * escl_thread_getspecific(escl_thread_key_t key)
 *
 *  Description :
 *  -------------
 *  The escl_thread_getspecific() function shall return the value
 *  currently bound to the specified key on behalf of the calling thread.
 *
 *  Parameter     :
 *  -----------
 *  key         :    key obtained via a previous call to escl_thread_key_create().
 *  Return values :
 *  ---------------
 *  the thread-specific data value associated with the given key.
 *  If no thread-specific data value is associated with key, then the
 *  value NULL shall be returned.
 *
 ************************************************************************/
#define escl_thread_getspecific(key) pthread_getspecific(key)

/************************************************************************
 *
 *    int escl_thread_getspecific(escl_thread_key_t key, const void *value)
 *
 *  Description :
 *  -------------
 *  The escl_thread_setspecific() function shall associate a thread-specific
 *  value with a key obtained via a previous call to escl_thread_key_create().
 *  Different threads may bind different values to the same key.
 *  These values are typically pointers to blocks of dynamically allocated memory
 *  that have been reserved for use by the calling thread.
 *
 *  Parameter     :
 *  -----------
 *  key         :    key obtained via a previous call to escl_thread_key_create().
 *  value       :    value to be set to thread local storage.
 *  Return values :
 *  ---------------
 *  If successful, returns zero; otherwise, an error number shall be returned to indicate the error.
 *
 ************************************************************************/
#define escl_thread_setspecific(key, value) pthread_setspecific(key, value)

/************************************************************************
 *
 *       void escl_semaphore_init(escl_semaphore_t *sem, unsigned val)
 *
 *  Description :
 *  -------------
 *  initialises the unnamed semaphore at the address pointed
 *  to by <sem>. The <val> argument specifies the initial value for the semaphore.
 *
 *  Parameter     :
 *  -----------
 *  sem         :    Pointer to semaphore object.
 *  val         :    Initial value.
 *  Return values :
 *  ---------------
 *  None
 *
 ************************************************************************/

#define escl_semaphore_init(sem, val) sem_init(sem, 0, val)

/************************************************************************
 *
 *          void escl_semaphore_destroy(escl_semaphore_t *sem)
 *
 *  Description :
 *  -------------
 *  Destroys the unnamed semaphore at the address pointed to by <sem>.
 *  Only a semaphore that has been initialised by escl_semaphore_init()
 *  should be destroyed using escl_semaphore_destroy().
 *
 *  Parameter     :
 *  -----------
 *  sem           :    Pointer to semaphore object.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_semaphore_destroy(sem) sem_destroy(sem)
// ncrements (unlocks) the semaphore pointed to by sem. If the semaphore's value consequently becomes greater than zero,
// then another process or thread blocked in a escl_semaphore_wait(3) call will be woken up and proceed to lock the
// semaphore.
#define escl_semaphore_post(sem) sem_post(sem)
#define escl_semaphore_wait(sem) sem_wait(sem)
#define escl_semaphore_trywait(sem) (sem_trywait(sem) == 0)
#define escl_semaphore_getvalue(sem, rval) sem_getvalue(sem, rval)
extern int escl_semaphore_timed_wait(escl_semaphore_t *sem, unsigned msecs);

#define escl_mlock(addr, length) mlock(addr, length)
#define escl_getpagesize() getpagesize()
#define escl_aligned_malloc(sz, align) memalign(align, sz)
#define escl_aligned_free(p) free(p)

/************************************************************************
 *
 *                void escl_thread_delay(unsigned msecs)
 *
 *  Description :
 *  -------------
 *  Suspends execution of the calling thread for (at least) msecs microseconds.
 *  The sleep may be lengthened slightly by any system activity or by the time
 *  spent processing the call or by the granularity of system timers.
 *
 *  Parameter :
 *  -----------
 *  msecs         :          time for thread to sleep (in microseconds).
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#define escl_thread_delay(msecs) usleep((msecs)*1000)

/************************************************************************
 *
 *                 void escl_thread_suspend(escl_thread_t tid)
 *  Description :
 *  -------------
 *  Postpones the execution of a thread. If a thread is suspended multiple times, the resume
 *  function call must be made for each suspend function call before the thread will execute.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/
#if defined(HAVE_PTHREAD_SUSPEND)
#define escl_thread_suspend(tid) pthread_suspend(tid)  // UNIX
#elif defined(HAVE_PTHREAD_SUSPEND_NP)
#define escl_thread_suspend(tid) pthread_suspend_np(tid)
#else                                          //! HAVE_PTHREAD_SUSPEND
#define escl_thread_suspend(tid) __ESCL_NOP__  // TODO *  might use pthread_cond_signal to implement.
#endif                                         // HAVE_PTHREAD_SUSPEND

/************************************************************************
 *
 *               void escl_thread_resume(escl_thread_t tid)
 *  Description :
 *  -------------
 *  Causes a thread to continue execution. This call must be made after a thread is created in
 *  order to start the execution of the thread. Each suspend function call must correspond to a
 *  resume function call before a thread will continue execution.
 *  Parameter :
 *  -----------
 *  tid         :       ID to the thread.
 *  Return values :
 *  ---------------
 *  N/A
 *
 ************************************************************************/

#if defined(HAVE_PTHREAD_RESUME)
#define escl_thread_resume(tid) pthread_resume(tid)
#elif defined(HAVE_PTHREAD_RESUME_NP)
#define escl_Thread_resume(tid) pthread_resume_np(tid)
#else                                         // HAVE_PTHREAD_RESUME
#define escl_thread_resume(tid) __ESCL_NOP__  // TODO * might use pthread_cond_signal to implement.
#endif                                        // HAVE_PTHREAD_RESUME

#if !defined(__CYGWIN__)
// Type definition of spin lock(POSIX).
typedef pthread_spinlock_t escl_spinlock_t;
#define escl_spinlock_init(x) pthread_spin_init((x), PTHREAD_PROCESS_PRIVATE)
#define escl_spinlock_destroy(x) pthread_spin_destroy((x))
#define escl_spinlock_lock(x) pthread_spin_lock((x))
#define escl_spinlock_unlock(x) pthread_spin_unlock((x))
#define escl_spinlock_trylock(x) (pthread_spin_trylock((x)) == 0)
#define escl_spinlock_lock_irqsave(x, f) escl_spinlock_lock(x)
#define escl_spinlock_unlock_irqrestore(x, f) escl_spinlock_unlock(x)

#define wake_up(w)                         \
    ({                                     \
        escl_mutex_lock(&w->lock);         \
        escl_thread_cond_signal(&w->cond); \
        escl_mutex_unlock(&w->lock);       \
    })

#define escl_init_waitqueue_head(w)                    \
    ({                                                 \
        escl_mutex_init(&(w)->lock);                   \
        escl_thread_cond_init(&(w)->cond, &(w)->lock); \
    })
#else /*__CYGWIN__*/
// Type definition of spin lock(POSIX).
typedef pthread_mutex_t escl_spinlock_t;
#define escl_spinlock_init(x) pthread_mutex_init((x), NULL)
#define escl_spinlock_destroy(x) pthread_mutex_destroy((x))
#define escl_spinlock_lock(x) pthread_mutex_lock((x))
#define escl_spinlock_unlock(x) pthread_mutex_unlock((x))
#define escl_spinlock_trylock(x) (pthread_mutex_trylock((x)) == 0)
#define escl_spinlock_lock_irqsave(x, f) escl_mutex_lock(x)
#define escl_spinlock_unlock_irqrestore(x, f) escl_mutex_unlock(x)

#define wake_up(w)                         \
    ({                                     \
        escl_mutex_lock(&w->lock);         \
        escl_thread_cond_signal(&w->cond); \
        escl_mutex_unlock(&w->lock);       \
    })

#define escl_init_waitqueue_head(w)                    \
    ({                                                 \
        escl_mutex_init(&(w)->lock);                   \
        escl_thread_cond_init(&(w)->cond, &(w)->lock); \
    })
#endif  //__CYGWIN__

extern int escl_thread_cond_init(escl_cond_t *cv, escl_mutex_t *mutex);
extern void escl_thread_cond_destroy(escl_cond_t *cv);
extern int escl_thread_cond_timed_wait(escl_cond_t *cv, escl_mutex_t *mutex, unsigned msecs);
extern int escl_thread_cond_wait(escl_cond_t *cv, escl_mutex_t *mutex);
extern int escl_thread_cond_signal(escl_cond_t *cv);
extern int escl_thread_cond_broadcast(escl_cond_t *cv);

/************************************************************************
 *
 *                   void escl_thread_yield(void)
 *  Description :
 *  -------------
 *   The escl_thread_yield() function resets the accumulated time slice of the
 *   current thread and then enters the thread scheduler, which resumes execu-
 *   tion of the next thread ready to run.  If no other thread is ready to
 *   run, control returns to the current thread.
 *
 *  Parameter :
 *  -----------
 *   N/A
 *
 *  Return values:
 *  ---------------
 *   N/A
 *
 ************************************************************************/

#if defined(__CYGWIN__)
#define escl_thread_yield() sleep(0)  // sched_yield() //pthread_sched()
#else                                 //!__CYGWIN__
#define escl_thread_yield() pthread_yield()
#endif  //__CYGWIN__

#endif  //!__KERNEL__

#elif defined(ESCL_BOOTLOADER)
#define escl_thread_delay(msecs) mdelay(msecs)
#define escl_thread_yield() __asm("nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;")
/*TODO*/
#else
#error "OS isn't supported."
#endif

/************************************************************************
 *
 *                      int escl_thread_create
 *                         (
 *                          escl_thread_t        *tid,
 *                          escl_thread_func_t    func,
 *                          escl_thread_data_t    arg,
 *                          unsigned             stack_size
 *                          )
 *  Description :
 *  -------------
 *   The escl_thread_create() function is used to create a new thread, with
 *   stack size specified by stack_size(The default size was defined by
 *   ESCL_THREAD_STACK_SIZE), Upon successful completion, escl_thread_create()
 *   will store the ID of the created thread in the location specified by tid.
 *
 *   Upon its creation, the thread executes func, with arg as its
 *   sole argument.  If start_routine returns, the effect is as if there was
 *   an implicit call return instruction,
 *
 *  Parameter :
 *  -----------
 *  tid        :    returned handle to the thread,
 *                  This pointer indicates where the new thread id should be put into.
 *  func       :    Routine that begins execution of the thread.
 *  arg        :    data value passed to the thread entry routine.
 *  stack_size :    Size, In bytes, of the stack for the thread.
 *  ---------------
 *  If successful, the escl_thread_create() function will return zero.
 *  Otherwise, an error number will be returned to indicate the error.
 *
 ************************************************************************/

// extern int escl_thread_create(escl_thread_t * tid, escl_thread_func_t func, escl_thread_data_t arg, unsigned
// stack_size);

/************************************************************************
 *  Implementation : Public functions
 ************************************************************************/

#if defined(__cplusplus)
};     /*if __cplusplus*/
#endif /*!__cplusplus*/

#endif  //__ESCL_OS_H__
