// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESCL_COMMON_STATUS_H__
#define __ESCL_COMMON_STATUS_H__

#include "escl_type.h"

typedef union {
    struct esclVERSION_NUM_S {
        escl_u16 patch;
        escl_u16 minor;
        escl_u32 major;
    } VERSION_NUM_S;
    escl_u64 version;
} ESCL_VERSION_U;

typedef escl_s32 esclRet;

#define ESCL_SUCCESS 0

/* 1010 0000b
 * VTOP use APPID from 0~31
 * so, eswin use APPID based on 32
 */
#define ESCL_ERR_APPID (0x80000000L + 0x20000000L)

typedef enum esclMOD_ID_E {
    ESCL_ID_VB = 1,
    ESCL_ID_SYS = 2,
    ESCL_ID_VDEC = 3,
    ESCL_ID_VPS = 4,
    ESCL_ID_VENC = 5,
    ESCL_ID_VO = 6,
    ESCL_ID_VI = 7,
    ESCL_ID_AIO = 8,
    ESCL_ID_AI = 9,
    ESCL_ID_AO = 10,
    ESCL_ID_AENC = 11,
    ESCL_ID_ADEC = 12,
    ESCL_ID_USER = 13,
    ESCL_ID_GDC = 14,
    ESCL_ID_NPU = 15,
    ESCL_ID_DSP = 16,
    ESCL_ID_BMS = 17,
    ESCL_ID_NUMA = 18,
    ESCL_ID_CIPHER = 19,
    ESCL_ID_AK = 20,
    ESCL_ID_MEMCP = 21,
    ESCL_ID_RPC = 22,
    ESCL_ID_PLATFORM = 23,
    ESCL_ID_SERVER = 24,
    ESCL_ID_MEMORY = 25,
    ESCL_ID_BUTT,
} ESCL_MOD_ID_E;

typedef enum {
    ESCL_LOG_LEVEL_ASSERT = 0,
    ESCL_LOG_LEVEL_ERROR = 1,
    ESCL_LOG_LEVEL_WARN = 2,
    ESCL_LOG_LEVEL_INFO = 3,
    ESCL_LOG_LEVEL_DEBUG = 4,
    ESCL_LOG_LEVEL_VERBOSE = 5,
    ESCL_LOG_LEVEL_BUTT
} ESCL_LOG_LEVEL_E;

typedef enum {
    ESCL_ERR_LEVEL_DEBUG = 0,   /* debug-level */
    ESCL_ERR_LEVEL_INFO = 1,    /* informational */
    ESCL_ERR_LEVEL_WARNING = 2, /* warning conditions */
    ESCL_ERR_LEVEL_ERROR = 3,   /* error conditions */
    ESCL_ERR_LEVEL_BUTT
} ESCL_ERR_LEVEL_E;

/******************************************************************************
|----------------------------------------------------------------|
| 1 |   APP_ID   |   MOD_ID    | ERR_LEVEL |   ERR_ID            |
|----------------------------------------------------------------|
|<--><--7bits----><----8bits---><--3bits---><------13bits------->|
******************************************************************************/

#define ESCL_DEF_ERR(module, level, errid) ((escl_s32)((ESCL_ERR_APPID) | ((module) << 16) | ((level) << 13) | (errid)))

#ifndef NULL
#define NULL 0L
#endif

#endif