/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_TDE_TDEIF_SERVER_H_)
#define _TDE_TDEIF_SERVER_H_

#include "tde_TdeIf_common.h"
#include "escl_rpc.h"
#include "esclrt.h"
#include "tde_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif

#if !defined(ESRPC_FUNCTIONS_DEFINITIONS_TDEIF)
#define ESRPC_FUNCTIONS_DEFINITIONS_TDEIF


/*! @brief TdeIf identifiers */
enum _TdeIf_ids
{
    kTdeIf_service_id = 2,
    kTdeIf_escltdeCreateHandle_id = 1,
    kTdeIf_escltdeDestoryHandle_id = 2,
    kTdeIf_escltdeFill_id = 3,
    kTdeIf_escltdeComplex_id = 4,
    kTdeIf_escltdeMultisource_id = 5,
    kTdeIf_escltdeLogo_id = 6,
};

/*! Manager interface */
//! @name TdeIf
//@{
/**
* \brief Create a tde handle. \n
* This function require the creation of a tde handle. \n
* \ingroup common_group
* \param [out]  handle          Point to a tde handle.
*/
escltdeStatus escltdeCreateHandle(uintptr_t * handle, const escltdeInitInfo * init_params);

/**
* \brief Destory a tde handle. \n
* This function require the creation of a tde handle. \n
* \ingroup common_group
* \param [in]  handle          Point to a tde handle.
*/
escltdeStatus escltdeDestoryHandle(uintptr_t handle);

/**
* \brief Tde fill a zone with color that was set by user. \n
* This function require the creation of a tde handle. \n
* \ingroup common_group
* \param [in]  handle            Point to a tde handle.
* \param [in]  buf_info          Tde buffer info.
* \param [in]  rect_info_array   The rect infos which was use to fill.
*/
escltdeStatus escltdeFill(uintptr_t handle, const escltdeBufInfo * buf_info, const escltdeRectInfoList * rect_info_array);

/**
* \brief Handling tde complex functions. \n
* This function require the creation of a tde handle. \n
* \ingroup common_group
* \param [in]     handle            Point to a tde handle.
* \param [in]     src_info          Src tde biffer info.
* \param [in]     dst_info          Dst tde biffer info.
* \param [in]     pat_info          Pat tde biffer info.
* \param [in]     src_rect          Source rect.
* \param [in]     dst_rect          Dst rect.
* \param [in]     pat_rect          Pat rect.
* \param [in]     src_disp_rect     Source disp rect.
* \param [in]     usage             Some flag, such as alphblend mode.
*/
escltdeStatus escltdeComplex(uintptr_t handle, const escltdeBufInfo * src_info, const escltdeBufInfo * dst_info, const escltdeBufInfo * pat_info, const escltdeRect * src_rect, const escltdeRect * dst_rect, const escltdeRect * pat_rect, const escltdeRect * src_disp_rect, int32_t usage);

/**
* \brief Handling tde multisource functions. \n
* This function require the creation of a tde handle. \n
* \ingroup common_group
* \param [in]     handle            Point to a tde handle.
* \param [in]     src_info          Src tde biffer info.
* \param [in]     src_rects         Source rect array.
* \param [in]     dst_rects         Dst rect array.
* \param [in]     dst_info          Dst tde biffer info.
*/
escltdeStatus escltdeMultisource(uintptr_t handle, const escltdeBufInfoList * src_info, const escltdeRectList * src_rects, const escltdeRectList * dst_rects, const escltdeBufInfo * dst_info);

/**
* \brief Handling tde logo functions. \n
* This function require the creation of a tde handle. \n
* \ingroup common_group
* \param [in]     handle            Point to a tde handle.
* \param [in]     src_info          Src tde biffer info.
* \param [in]     dst_info          Dst tde biffer info.
* \param [in]     src_rect          Source rect.
* \param [in]     src_disp_rect     Source disp rect.
* \param [in]     usage             Some flag, such as alphblend mode.
*/
escltdeStatus escltdeLogo(uintptr_t handle, const escltdeBufInfo * src_info, const escltdeBufInfo * dst_info, const escltdeRect * src_rect, const escltdeRect * src_disp_rect, int32_t usage);
//@}


#endif // ESRPC_FUNCTIONS_DEFINITIONS_TDEIF

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Initialize rpc service for TdeIf.
 */
void init_TdeIf(void);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Handle rpc request for TdeIf.
 */
void handle_TdeIf(rpc_s_ctx_t rpc, uint32_t stream_id, void *request, uint32_t request_size);

#if defined(__cplusplus)
}
#endif

#endif // _TDE_TDEIF_SERVER_H_
