// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESCL_HIGH_RES_TIMER_H
#define ESCL_HIGH_RES_TIMER_H

#include "escl_type.h"
#include <stdint.h>

#ifdef __cplusplus
#include <chrono>
#include <string>
#include <memory>
#include <cstring>

extern "C" {
escl_u64 escl_get_current_time_ns()
{
    using namespace std::chrono;
    return duration_cast<nanoseconds>(high_resolution_clock::now().time_since_epoch()).count();
}
escl_u64 escl_get_current_time() { return (escl_u64)escl_get_current_time_ns() / 1000; }
#else
escl_u64 escl_get_current_time_ns()
{
    struct timespec ts;
    clock_gettime(CLOCK_MONOTONIC, &ts);
    uint64_t nanoseconds = (uint64_t)ts.tv_sec * UINT64_C(1000000000) + (uint64_t)ts.tv_nsec;
    return nanoseconds;
}
escl_u64 escl_get_current_time() { return (escl_u64)escl_get_current_time_ns() / 1000; }
#endif

escl_u64 escl_get_elapsed_time_ms(escl_u64 start, escl_u64 end) { return (end - start) / 1e3; }
escl_u64 escl_get_elapsed_time_us(escl_u64 start, escl_u64 end) { return (end - start); }

#ifdef __cplusplus
}
#endif

#endif  // ESCL_HIGH_RES_TIMER_H