// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESCL_PERF_TRACER_H
#define ESCL_PERF_TRACER_H

#include "escl_type.h"

#ifdef __cplusplus
extern "C" {
#endif

// 初始化性能追踪器
void escl_trace_init(escl_s32 enable, const char* output_path);

// 记录函数开始 (C兼容接口)
void escl_trace_start(const char* tag_name);

// 记录函数结束 (C兼容接口)
void escl_trace_end(const char* tag_name);
void escl_trace_insert(const char* tag_name, escl_u64 starttime, escl_u64 endtime);
// 写入追踪数据
void escl_trace_flush();

#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
#include <memory>
#include <vector>
#include <mutex>
#include <fstream>
#include <chrono>
#include <thread>
#include <memory>
#include <atomic>
#include <string>
#include "escl_trace_lfvector.h"

class ESCLPerfTracer
{
 public:
    class TraceEvent
    {
     public:
        std::string name;
        char ph;
        escl_u64 ts;
        escl_u32 tid;
        TraceEvent() = default;
        TraceEvent(const TraceEvent&) = default;
        TraceEvent& operator=(const TraceEvent&) = default;
    };

    // RAII辅助类
    class EndEventGuard
    {
     public:
        EndEventGuard(const char* tag_name) : tag_name_(tag_name) {}
        ~EndEventGuard() { GetInstance().EndEvent(tag_name_); }

     private:
        const char* tag_name_;
    };

    static ESCLPerfTracer& GetInstance();

    void Init(escl_s32 enable, const std::string& output_path);
    void BeginEvent(const char* tag_name);
    void EndEvent(const char* tag_name);
    void Flush();
    void InsertEvent(const char* tag_name, escl_u64 startTime, escl_u64 endTime);

 private:
    static escl_u32 GetThreadId();
    std::mutex mutex_;
    std::string output_path_;
    escl_s32 enable_ = 0;
    LockFreeVector<TraceEvent> events_;
};
#endif

#endif  // ESCL_PERF_TRACER_H