// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESCL_TYPE_H__
#define __ESCL_TYPE_H__
#include <stdbool.h>
#include <stdint.h>
#include "escl_platform.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if defined(WIN32) || defined(WINDOWS)

typedef unsigned __int64 u64_t;
typedef unsigned __int32 u32_t;
typedef unsigned __int16 u16_t;
typedef unsigned __int8 u8_t;
typedef signed __int64 s64_t;
typedef signed __int32 s32_t;
typedef signed __int16 s16_t;
typedef signed __int8 s8_t;

#else /*!WINDOWS*/

typedef uint_least64_t u64_t;
typedef uint_least32_t u32_t;
typedef uint_least16_t u16_t;
typedef uint_least8_t u8_t;

typedef int_least64_t s64_t;
typedef int_least32_t s32_t;
typedef int_least16_t s16_t;
typedef int_least8_t s8_t;

#endif /*WINDOWS*/

/**< backward compatible */
typedef s64_t i64_t;
typedef s32_t i32_t;
typedef s16_t i16_t;
typedef s8_t i8_t;

typedef u64_t escl_u64;
typedef u32_t escl_u32;
typedef u16_t escl_u16;
typedef u8_t escl_u8;

typedef s64_t escl_s64;
typedef s32_t escl_s32;
typedef s16_t escl_s16;
typedef s8_t escl_s8;

typedef s64_t escl_i64;
typedef s32_t escl_i32;
typedef s16_t escl_i16;
typedef s8_t escl_i8;

/**< Storage of a pointer */
typedef unsigned long escl_ulong;
/**< signed Result of sizeof operator.*/
#if defined(_MSC_VER)
typedef SSIZE_T escl_ssize_t;
#else  /*_MSC_VER*/
typedef ssize_t escl_ssize_t;
#endif /*_MSC_VER*/
/**<Result of sizeof operator.*/
typedef size_t escl_size_t;
/**<Pointer for both 64 or 32 bit wide.*/
typedef intptr_t escl_intptr_t;
typedef uintptr_t escl_uintptr_t;
/**<Result of subtraction of two pointers.*/
typedef ptrdiff_t escl_ptrdiff_t;
/**<Represents file-offset value.*/
typedef off_t escl_off_t;
/**<Used for uniquely specifying every position within a file.*/
typedef fpos_t escl_fpos_t;
/**<
 *the number of seconds since January 1, 1970, 0:00 UTC.
 *If _USE_32BIT_TIME_T is defined, time_t is a long integer.
 *If not defined, it is a 64-bit integer.
 */
typedef time_t escl_time_t;
/**< capable of representing any value of any signed integer type.*/
typedef intmax_t escl_intmax_t;
/**< capable of representing any value of any unsigned integer type.*/
typedef uintmax_t escl_uintmax_t;
/**<Useful for writing portable programs for international markets*/
typedef wchar_t escl_wchar_t;
/**<Can represent all characters of any language character set.*/
typedef wctype_t escl_wctype_t;

#if defined(__cplusplus)
typedef bool escl_bool_t;
typedef bool bool_t;
#else /*!__cplusplus*/
typedef unsigned escl_bool_t;
typedef unsigned bool_t;
#ifndef true
#define true (1 == 1)
#endif /*true*/
#ifndef false
#define false (!true)
#endif /*false*/
#endif /*__cplusplus*/

#ifdef __cplusplus
}
#endif /* __cplusplus */

#include "escl_status.h"

#endif /* __ESCL_TYPE_H__ */
