// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESCL_COMMON_UTILS_H__
#define __ESCL_COMMON_UTILS_H__

#if defined(__cplusplus)
#ifndef __ExternC
#define __ExternC extern "C"
#endif /*__ExternC*/
#ifndef __EXTERNC_BEGIN
#define __EXTERNC_BEGIN extern "C" {
#endif /*__EXTERNC_BEGIN*/
#ifndef __EXTERNC_END
#define __EXTERNC_END }
#endif /*__EXTERNC_END*/
#else  /*!__cplusplus */
#ifndef __ExternC
#define __ExternC extern
#endif /*__ExternC */
#ifndef __EXTERNC_BEGIN
#define __EXTERNC_BEGIN
#endif /*__EXTERNC_BEGIN */
#ifndef __EXTERNC_END
#define __EXTERNC_END
#endif /*__EXTERNC_END */
#endif /*__cplusplus*/

#ifndef HIGHWORD
/**
 * @def    HIGHWORD
 * @brief     Macro to    extract    the    high word    of a long    offset
 * @param[in]    l    address    to write
 */
#define HIGHWORD(l) ((u16_t)(((u32_t)(l) >> 16) & 0xffff))
#endif /*    HIGHWORD */

#ifndef LOWWORD
/**
 * @def    LOWWORD
 * @brief     Macro to    extract    the    low    word of    a    long offset
 * @param[in]    l    address    to write
 */
#define LOWWORD(l) ((u16_t)((u32_t)l & 0x0000ffff))
#endif /*    LOWWORD    */

// Macro to    combine    two    USHORT offsets into    a    long offset
#ifndef MAKEULONG
/**
 * @def    MAKEULONG
 * @brief     Macro to    combine    two    USHORT offsets into    a    long offset
 * @param[in]    x    data to    combine, high    16 bits
 * @param[in]    y    data to    combine, low 16    bits
 */
#define MAKEULONG(x, y) (((((u32_t)(x)) << 16) & 0xffff0000) | ((u32_t)(y)&0xffff))
#endif /*    MAKEULONG    */

/************************************************************************
 *    Misc Definitions
 *************************************************************************/
#ifndef TRUE
#define TRUE (1 == 1)
#endif /*TRUE*/
#ifndef FALSE
#define FALSE (!(1 == 1))
#endif /*FALSE*/
#ifndef NULL
#define NULL ((void *)0)
#endif /*NULL*/
#ifndef nullPtr
#define nullPtr NULL
#endif /*nullPtr*/

#ifndef ESCL_BIG_ENDIAN
/**
 * @def    ESCL_BIG_ENDIAN
 * @brief     big endian    MACRO
 */
#define ESCL_BIG_ENDIAN 0
#endif /*ESCL_BIG_ENDIAN*/

#ifndef ESCL_LITTLE_ENDIAN
/**
 * @def    ESCL_LITTLE_ENDIAN
 * @brief     little    endian
 */
#define ESCL_LITTLE_ENDIAN 1
#endif /*ESCL_LITTLE_ENDIAN*/

#ifndef ESCL_DOUBLE_PRECISION_VALUE_NAN
/**
 * @def    ESCL_DOUBLE_PRECISION_VALUE_NAN
 * @brief     Float-to-integer    conversions    generate Invalid Operation exceptions    rather than    Overflow or
 * Underflow exceptions. To    support    signed conversions with    round-towards-zero rounding in    the    maximum
 * range    possible for C,    C++, and Java    compiled code, the thresholds    for    pessimistic bouncing    are
 * different    for    the    various    rounding modes. Below shows the single-precision    float-to-integer bounce
 * range    and    the    results    returned for exceptional conditions. The exponent    values shown in Table    5.12
 * are in    biased format.
 *
 */
#define ESCL_DOUBLE_PRECISION_VALUE_NAN 0x0000000000000000
#endif /*ESCL_DOUBLE_PRECISION_VALUE_NAN*/

#ifndef ESCL_DOUBLE_PRECISION_VALUE_INFINITY
/**
 * @def    ESCL_DOUBLE_PRECISION_VALUE_INFINITY
 * @brief     Float-to-integer    conversions    generate Invalid Operation exceptions    rather than    Overflow or
 * Underflow exceptions. To    support    signed conversions with    round-towards-zero rounding in    the    maximum
 * range    possible for C,    C++, and Java    compiled code, the thresholds    for    pessimistic bouncing    are
 * different    for    the    various    rounding modes. Below shows the single-precision    float-to-integer bounce
 * range    and    the    results    returned for exceptional conditions. The exponent    values shown in Table    5.12
 * are in    biased format.
 *
 */
#define ESCL_DOUBLE_PRECISION_VALUE_INFINITY 0x7FF0000000000000
#endif /*ESCL_DOUBLE_PRECISION_VALUE_INFINITY*/

#ifndef ESCL_SINGLE_PRECISION_VALUE_NAN
/**
 * @def    ESCL_SINGLE_PRECISION_VALUE_NAN
 * @brief     Float-to-integer    conversions    generate Invalid Operation exceptions    rather than    Overflow or
 * Underflow exceptions. To    support    signed conversions with    round-towards-zero rounding in    the    maximum
 * range    possible for C,    C++, and Java    compiled code, the thresholds    for    pessimistic bouncing    are
 * different    for    the    various    rounding modes. Below shows the single-precision    float-to-integer bounce
 * range    and    the    results    returned for exceptional conditions. The exponent    values shown in Table    5.12
 * are in    biased format.
 *
 */
#define ESCL_SINGLE_PRECISION_VALUE_NAN 0x00000000
#endif /*ESCL_SINGLE_PRECISION_VALUE_NAN*/

#ifndef ESCL_SINGLE_PRECISION_VALUE_INFINITY
/**
 * @def    ESCL_SINGLE_PRECISION_VALUE_NAN
 * @brief     Float-to-integer    conversions    generate Invalid Operation exceptions    rather than    Overflow or
 * Underflow exceptions. To    support    signed conversions with    round-towards-zero rounding in    the    maximum
 * range    possible for C,    C++, and Java    compiled code, the thresholds    for    pessimistic bouncing    are
 * different    for    the    various    rounding modes. Below shows the single-precision    float-to-integer bounce
 * range    and    the    results    returned for exceptional conditions. The exponent    values shown in Table    5.12
 * are in    biased format.
 *
 */
#define ESCL_SINGLE_PRECISION_VALUE_INFINITY 0x7F7FFFFF
#endif /*ESCL_SINGLE_PRECISION_VALUE_INFINITY*/

#if defined(__GNUC__)
#ifndef __packed
#define __packed __attribute__((packed)) /* byte aligned.*/
#endif                                   /* __packed    */
#ifndef __unused
#define __unused __attribute__((unused)) /* Avoids warning for unused    variable and function.*/
#endif                                   /* __unused    */
#ifndef __align_8
#define __align_8 __attribute__((aligned(8))) /* 8 bytes aligned.*/
#endif                                        /*__align_8    */
#ifndef __align_16
#define __align_16 __attribute__((aligned(16))) /* 16    bytes    aligned.*/
#endif                                          /*__align_16 */
#ifndef __align_32
#define __align_32 __attribute__((aligned(32))) /* 32    bytes    aligned.*/
#endif                                          /*__align_32 */
#ifndef __weak
#define __weak __attribute__((weak)) /* Weak symbol.*/
#endif                               /*__weak*/
#ifndef __format_printf
#define __format_printf(n, m) __attribute__((format(printf, n, m)))
#endif  //__format_printf
#ifndef __format_scanf
#define __format_scanf(n, m) __attribute__((format(scanf, n, m)))
#endif  //__format_scanf
#ifndef __calltype
#if defined(__SDE_MIPS__)
#define __calltype
#else                                     //__SDE_MIPS__
#define __calltype __attribute__((cdecl)) /*Call convention.*/
#endif                                    //__SDE_MIPS__
#endif                                    /*__calltype */
#define __ESCL_PACKED_PUSH(n)
#define __ESCL_PACKED_POP()
#define __escl_compiler_barrier__() __sync_synchronize()
#if defined(__KERNEL__)
#define __escl_wmb__() wmb()
#define __escl_rmb__() rmb()
#else /*__KERNEL__*/
#if (LINUX_VERSION_CODE >= 0x040300)
#define __escl_wmb__() membarrier(MEMBARRIER_CMD_GLOBAL, 0)
#define __escl_rmb__() membarrier(MEMBARRIER_CMD_GLOBAL, 0)
#else /*#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 3, 0))*/
#define __escl_wmb__() \
    do {               \
    } while (0)
#define __escl_rmb__() \
    do {               \
    } while (0)
#endif /*#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 3, 0))*/
#endif /*__KERNEL__*/
#else  /*__GNUC__    is not defined */
#ifndef __packed
#define __packed /* #pragma        pack(1) */
#endif           /* __packed    */
#ifndef __unused
#define __unused
#endif /* __unused    */
#ifndef __align_8
#define __align_8 /* #pragma     pack(8)*/
#endif            /*__align_8    */
#ifndef __align_16
#define __align_16
#endif /*__align_16 */
#ifndef __align_32
#define __align_32
#endif /*__align_32 */
#ifndef __format_scanf
#define __format_scanf(n, m)
#endif /*__format_scanf*/
#ifndef __format_printf
#define __format_printf(n, m)
#endif /*__format_printf*/
#ifndef __weak
#define __weak
#endif /*__weak*/
#ifndef __calltype
#define __calltype __stdcall
#endif /*__calltype */

#if !defined(LINUX) && !defined(__linux) && !defined(__CYGWIN__) && !defined(WIN32) && !defined(WINDOWS)
#define __ESCL_PACKED_PUSH(n) #pragma pack(push, n)
#define __ESCL_PACKED_POP() #pragma pack(pop)
#endif /*!(LINUX)    && !(__linux) &&    !(__CYGWIN__) &&    !(WIN32)    && !(WINDOWS)*/
#define __escl_compiler_barrier__() MemoryBarrier()
#define __escl_wmb__() _WriteBarrier()
#define __escl_rmb__() _ReadBarrier()
#endif /*__GNUC__*/

#ifndef ESCL_INVALID_VALUE
#if defined(__x86_64__) || defined(__ppc64__) || defined(_WIN64) || defined(__arch64__)
#define ESCL_INVALID_VALUE ((u64_t)-1)
#else /*not defined(__x86_64__) || defined(__ppc64__) || defined(_WIN64) || defined(__arch64__)*/
#define ESCL_INVALID_VALUE 0xffffffff
#endif /*not defined(__x86_64__) || defined(__ppc64__) || defined(_WIN64) || defined(__arch64__)*/
#endif /*ESCL_INVALID_VALUE*/

#ifndef IS_NOT_ALIGNED
#define IS_NOT_ALIGNED(addr, n) ((u32_t)(addr) & ((n)-1))
#endif /* IS_NOT_ALIGNED*/

#ifndef ALIGNED
#define ALIGNED(addr, n) (((u32_t)(addr) + ((n)-1)) & (~((n)-1)))
#define ALIGNED4(addr) ALIGNED(addr, 4)
#define ALIGNED8(addr) ALIGNED(addr, 8)
#define ALIGNED16(addr) ALIGNED(addr, 16)
#define ALIGNED32(addr) ALIGNED(addr, 32)
#endif /*ALIGNED*/

#ifndef ALIGN_DOWN
#define ALIGN_DOWN(length, type) ((u32_t)(length) & ~(sizeof(type) - 1))
#endif /*ALIGN_DOWN*/

#ifndef ALIGN_UP
#define ALIGN_UP(length, type) (ALIGN_DOWN(((u32_t)(length) + sizeof(type) - 1), type))
#endif /*ALIGN_UP*/

#ifndef ALIGN_DOWN_POINTER
#define ALIGN_DOWN_POINTER(address, size) ((u32_t)((u32_t)(address) & ~((u32_t)(size)-1)))
#endif /*ALIGN_DOWN_POINTER*/

#ifndef ALIGN_UP_POINTER
#define ALIGN_UP_POINTER(address, size) (ALIGN_DOWN_POINTER(((u32_t *)(address) + (size - 1)), size))
#endif /*ALIGN_UP_POINTER*/

#ifndef ROUND_DOWN
#define ROUND_DOWN(addr, boundary) ((u32_t)(addr) & ~((boundary)-1))
#endif /*ROUND_DOWN*/

#ifndef ROUND_UP
#define ROUND_UP(addr, boundary) (((u32_t)(addr) + (boundary)-1) & ~((boundary)-1))
#endif /*ROUND_UP*/

#ifndef ESCL_MIN
#define ESCL_MIN(a, b) ((a) > (b) ? (b) : (a))
#endif /*ESCL_MIN*/

#ifndef ESCL_MAX
#define ESCL_MAX(a, b) ((a) > (b) ? (a) : (b))
#endif /*ESCL_MAX*/

#ifndef __SWAB16
#define __SWAB16(x) ((((x)&0xFF) << 8) | (((x) >> 8) & 0xFF))
#endif /*__SWAB16*/

#ifndef __SWAB32
#define __SWAB32(x) ((((x)&0xff) << 24) | (((x)&0xff00) << 8) | (((x) >> 8) & 0xff00) | (((x) >> 24) & 0xff))
#endif /*    __SWAB32 */

#ifndef __SWAB64
#define __SWAB64(x)                                                                                        \
    ((((x) & (u64_t)(0xFF)) << 56) | (((x) & (u64_t)(0xFF00)) << 40) | (((x) & (u64_t)(0xFF0000)) << 24) | \
     (((x) & (u64_t)(0xFF000000)) << 8) | (((x) & (u64_t)(0xFF00000000)) >> 8) |                           \
     (((x) & (u64_t)(0xFF0000000000)) >> 24) | (((x) & (u64_t)(0xFF000000000000)) >> 40) |                 \
     (((x) & (u64_t)(0xFF00000000000000)) >> 56))
#endif /*__SWAB64*/

#if (ESCL_BYTEORDER) == (ESCL_BIG_ENDIAN)
#define Hton64(a) (a)
#define Ntoh64(a) (a)
#define Hton32(a) (a)
#define Ntoh32(a) (a)
#define Hton16(a) (a)
#define Ntoh16(a) (a)
#else /*ESCL_BYTE_ORDER != ESCL_BIT_ENDIAN*/
#define Hton64(a) __SWAB64(a)
#define Ntoh64(a) __SWAB64(a)
#define Hton32(a) __SWAB32(a)
#define Ntoh32(a) __SWAB32(a)
#define Hton16(a) __SWAB16(a)
#define Ntoh16(a) __SWAB16(a)
#endif /*ESCL_BYTEORDER    != ESCL_BIG_ENDIAN*/

#define bit_reverse_u32(x)                                                                                             \
    ((x) = (x) >> 16 | (0x0000ffff & (x)) << 16, (x) = (0xff00ff00 & (x)) >> 8 | (0x00ff00ff & (x)) << 8,              \
     (x) = (0xf0f0f0f0 & (x)) >> 4 | (0x0f0f0f0f & (x)) << 4, (x) = (0xcccccccc & (x)) >> 2 | (0x33333333 & (x)) << 2, \
     (x) = (0xaaaaaaaa & (x)) >> 1 | (0x55555555 & (x)) << 1)
#define bit_reverse_u8(x)                                                                      \
    ((x) = (0xf0 & (x)) >> 4 | (0x0f & (x)) << 4, (x) = (0xcc & (x)) >> 2 | (0x33 & (x)) << 2, \
     (x) = (0xaa & (x)) >> 1 | (0x55 & (x)) << 1)

/*It will    be got rid of    compiler. */
#ifndef __ESCL_NOP__
#define __ESCL_NOP__ \
    do {             \
    } while (0)
#endif /*__ESCL_NOP__ */

/*Infinite loop.*/
#ifndef __ESCL_INFI_LOOP__
#define __ESCL_INFI_LOOP__ while (1)
#endif /*__ESCL_INFI_LOOP__*/

/*For    C99    compliant.*/
#ifndef __ESCL_FUNC__
#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#define __ESCL_FUNC__ __func__
#elif defined(__GNUC__) /*__STDC_VERSION___*/ /*For    C89    compliant.*/
#define __ESCL_FUNC__ __FUNCTION__
#elif defined(_MSC_VER) && (_MSC_VER < 1300)
#define __FUNCTION__ __FILE__ /*not    supported    by vc6*/
#else                         /*__MSC_VER >= 1300 */
#define __ESCL_FUNC__ __func__
#endif /*__STDC_VERSION__*/
#endif /*__ESCL_FUNC__*/

#ifdef __ESCL_DEBUG__
#if defined(WIN32) || defined(WINDOWS)
#define ESCL_DBG_PRINT printf
#define ESCL_ASSERT(cond) assert(cond)
#elif defined(ESCL_BOOTLOADER)
#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#define ESCL_DBG_PRINT(fmt, args...) printf(fmt, __VA_ARGS__) /*C99    compliant*/
#else                                                         /*__STDC_VERSION__*/
#define ESCL_DBG_PRINT(fmt, args...) printf(fmt, ##args)
#endif /*__STDC_VERSION__*/
#define ESCL_ASSERT(cond)                                                                                      \
    {                                                                                                          \
        if (!(cond)) {                                                                                         \
            ESCL_DBG_PRINT("Assertion    is failed. file:%s,    line:%u, function:%s\r\n", __FILE__, __LINE__, \
                           __ESCL_FUNC__);                                                                     \
            __ESCL_INFI_LOOP__;                                                                                \
        }                                                                                                      \
    }
#elif defined(LINUX) || defined(__linux) || defined(__CYGWIN__)
#if defined(__KERNEL__) && !defined(__CYGWIN__)
#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#define ESCL_DBG_PRINT(fmt, args...) printk(fmt, __VA_ARGS__) /*C99    compliant*/
#else                                                         /*__STDC_VERSION__*/
#define ESCL_DBG_PRINT(fmt, args...) printk(fmt, ##args)
#endif /*__STDC_VERSION__*/
#define ESCL_ASSERT(cond)                                                                                      \
    {                                                                                                          \
        if (!(cond)) {                                                                                         \
            ESCL_DBG_PRINT("Assertion    is failed. file:%s,    line:%u, function:%s\r\n", __FILE__, __LINE__, \
                           __ESCL_FUNC__);                                                                     \
            panic("kernel    panic...\r\n");                                                                   \
        }                                                                                                      \
    }
#else /*!__KERNEL__*/
#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#define ESCL_DBG_PRINT(fmt, args...) fprintf(stderr, fmt, __VA_ARGS__) /*C99 compliant*/
#else                                                                  /*__STDC_VERSION__*/
#define ESCL_DBG_PRINT(fmt, args...) printf(fmt, ##args)               /*fprintf(stderr, fmt,    ##args)*/
#endif                                                                 /*__STDC_VERSION__*/
#define ESCL_ASSERT(cond) assert(cond)
#endif /*__KERNEL__*/
#elif defined(ECOS) || defined(__ECOS) || defined(__ECOS__) || defined(ESCL_BOOTLOADER)
#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#define ESCL_DBG_PRINT(fmt, args...) printf(fmt, __VA_ARGS__) /*C99    compliant*/
#else                                                         /*__STDC_VERSION__*/
#define ESCL_DBG_PRINT(fmt, args...) printf(fmt, ##args)
#endif /*__STDC_VERSION__*/
#define ESCL_ASSERT(cond)                                                                                      \
    {                                                                                                          \
        if (!(cond)) {                                                                                         \
            ESCL_DBG_PRINT("Assertion    is failed. file:%s,    line:%u, function:%s\r\n", __FILE__, __LINE__, \
                           __ESCL_FUNC__);                                                                     \
            __ESCL_INFI_LOOP__;                                                                                \
        }                                                                                                      \
    }
#else /*WIN32*/
#error "Unsupports OS."
#endif /*WIN32*/
#else  //!__ESCL_DEBUG__
#if defined(WIN32) || defined(WINDOWS)
#define ESCL_ASSERT(cond) __ESCL_NOP__
#define ESCL_DBG_PRINT(x) __ESCL_NOP__
#elif defined(ESCL_BOOTLOADER)
#define ESCL_ASSERT(cond) __ESCL_NOP__
#define ESCL_DBG_PRINT(fmt, args...) __ESCL_NOP__
#elif defined(LINUX) || defined(__linux) || defined(__CYGWIN__)
#if defined(__KERNEL__) && !defined(__CYGWIN__)
#define ESCL_ASSERT(cond) __ESCL_NOP__
#define ESCL_DBG_PRINT(fmt, args...) __ESCL_NOP__
#else /*!__KERNEL__*/
#define ESCL_ASSERT(cond) __ESCL_NOP__
#define ESCL_DBG_PRINT(fmt, args...) __ESCL_NOP__
#endif /*__KERNEL__*/
#elif defined(ECOS) || defined(__ECOS) || defined(__ECOS__) || defined(ESCL_BOOTLOADER)
#define ESCL_ASSERT(cond) __ESCL_NOP__
#define ESCL_DBG_PRINT(fmt, args...) __ESCL_NOP__
#else /*WIN32*/
#error "Unsupports OS."
#endif /*WIN32*/
#endif /*__ESCL_COMMON_UTILS_H__*/

#endif