/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_VDEC_VDECIF_SERVER_H_)
#define _VDEC_VDECIF_SERVER_H_

#include "vdec_VdecIf_common.h"
#include "escl_rpc.h"
#include "esclrt.h"
#include "vdec_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif

#if !defined(ESRPC_FUNCTIONS_DEFINITIONS_VDECIF)
#define ESRPC_FUNCTIONS_DEFINITIONS_VDECIF


/*! @brief VdecIf identifiers */
enum _VdecIf_ids
{
    kVdecIf_service_id = 3,
    kVdecIf_esclvdecDWLInit_id = 1,
    kVdecIf_esclvdecDWLRelease_id = 2,
    kVdecIf_esclvdecDWLReadAsicCoreCount_id = 3,
    kVdecIf_esclvdecDWLMallocLinear_id = 4,
    kVdecIf_esclvdecDWLFreeLinear_id = 5,
    kVdecIf_esclvdecDWLmemset_id = 6,
    kVdecIf_esclvdecVCDecMCGetCoreCount_id = 7,
    kVdecIf_esclvdecVCDecInit_id = 8,
    kVdecIf_esclvdecVCDecGetInfo_id = 9,
    kVdecIf_esclvdecVCDecSetInfo_id = 10,
    kVdecIf_esclvdecVCDecDecode_id = 11,
    kVdecIf_esclvdecVCDecNextPicture_id = 12,
    kVdecIf_esclvdecVCDecPictureConsumed_id = 13,
    kVdecIf_esclvdecVCDecEndOfStream_id = 14,
    kVdecIf_esclvdecVCDecGetBufferInfo_id = 15,
    kVdecIf_esclvdecVCDecAddBuffer_id = 16,
    kVdecIf_esclvdecVCDecAbort_id = 17,
    kVdecIf_esclvdecVCDecAbortAfter_id = 18,
    kVdecIf_esclvdecVCDecRelease_id = 19,
    kVdecIf_esclvdecUpdateInfoReport_id = 20,
};

/*! vdec interface */
//! @name VdecIf
//@{
/*!dwl interfaces*/
esclRet esclvdecDWLInit(const esclvdecDWLInitParam * param, uintptr_t * inst);

esclRet esclvdecDWLRelease(uintptr_t dwlinst);

uint32_t esclvdecDWLReadAsicCoreCount(uint32_t client_type);

esclRet esclvdecDWLMallocLinear(uintptr_t dwlinst, const esclvdecDWLLinearMemAllocInfo * alloc_info, esclvdecDWLLinearMem * mem_out);

void esclvdecDWLFreeLinear(uintptr_t dwlinst, const esclvdecDWLLinearMem * info);

uintptr_t esclvdecDWLmemset(uintptr_t d, int32_t c, uint32_t n);

/*! end of dwl interfaces */
uint32_t esclvdecVCDecMCGetCoreCount(void);

esclvdecDecRet esclvdecVCDecInit(const esclvdecDecInitConfig * init_config, esclvdecDecInst * pInst);

esclvdecDecRet esclvdecVCDecGetInfo(const esclvdecDecInst * pInst, esclvdecDecSequenceInfo * info);

esclvdecDecRet esclvdecVCDecSetInfo(const esclvdecDecInst * pInst, const esclvdecDecConfig * config);

esclvdecDecRet esclvdecVCDecDecode(const esclvdecDecInst * pInst, const esclvdecDecInputParameters * param, esclvdecDecOutput * output);

esclvdecDecRet esclvdecVCDecNextPicture(const esclvdecDecInst * pInst, esclvdecDecPicturePpu * pic);

esclvdecDecRet esclvdecVCDecPictureConsumed(const esclvdecDecInst * pInst, const esclvdecDecPicturePpu * pic);

esclvdecDecRet esclvdecVCDecEndOfStream(const esclvdecDecInst * pInst);

esclvdecDecRet esclvdecVCDecGetBufferInfo(const esclvdecDecInst * pInst, esclvdecDecBufferInfo * buf_info);

esclvdecDecRet esclvdecVCDecAddBuffer(const esclvdecDecInst * pInst, const esclvdecDWLLinearMem * buf);

esclvdecDecRet esclvdecVCDecAbort(const esclvdecDecInst * pInst);

esclvdecDecRet esclvdecVCDecAbortAfter(const esclvdecDecInst * pInst);

void esclvdecVCDecRelease(const esclvdecDecInst * pInst);

void esclvdecUpdateInfoReport(const esclvdecDecInst * pInst);
//@}


#endif // ESRPC_FUNCTIONS_DEFINITIONS_VDECIF

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Initialize rpc service for VdecIf.
 */
void init_VdecIf(void);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Handle rpc request for VdecIf.
 */
void handle_VdecIf(rpc_s_ctx_t rpc, uint32_t stream_id, void *request, uint32_t request_size);

#if defined(__cplusplus)
}
#endif

#endif // _VDEC_VDECIF_SERVER_H_
