/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_JENC_JENCIF_COMMON_H_)
#define _JENC_JENCIF_COMMON_H_
#include "jenc_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#if !defined(ESRPC_TYPE_DEFINITIONS_JENC)
#define ESRPC_TYPE_DEFINITIONS_JENC

// Enumerators data types declarations
enum
{
    MAX_JENC_STRM_BUF_NUM = 2,
    JECN_FILTER_NUM = 128
};

typedef enum escljencJpegEncRet
{
    ESCL_JPEGENC_FRAME_READY = 1,
    ESCL_JPEGENC_RESTART_INTERVAL = 2,
    ESCL_JPEGENC_OK = 0,
    ESCL_JPEGENC_ERROR = -1,
    ESCL_JPEGENC_NULL_ARGUMENT = -2,
    ESCL_JPEGENC_INVALID_ARGUMENT = -3,
    ESCL_JPEGENC_MEMORY_ERROR = -4,
    ESCL_JPEGENC_INVALID_STATUS = -5,
    ESCL_JPEGENC_OUTPUT_BUFFER_OVERFLOW = -6,
    ESCL_JPEGENC_EWL_ERROR = -7,
    ESCL_JPEGENC_EWL_MEMORY_ERROR = -8,
    ESCL_JPEGENC_HW_BUS_ERROR = -9,
    ESCL_JPEGENC_HW_DATA_ERROR = -10,
    ESCL_JPEGENC_HW_TIMEOUT = -11,
    ESCL_JPEGENC_HW_RESERVED = -12,
    ESCL_JPEGENC_SYSTEM_ERROR = -13,
    ESCL_JPEGENC_INSTANCE_ERROR = -14,
    ESCL_JPEGENC_HW_RESET = -15
} escljencJpegEncRet;

typedef enum escljencJpegEncFrameType
{
    ESCL_JPEGENC_YUV420_PLANAR = 0,                           /**< \brief  YYYY... UUUU... VVVV...  */
    ESCL_JPEGENC_YUV420_SEMIPLANAR = 1,                           /**< \brief  YYYY... UVUVUV...        */
    ESCL_JPEGENC_YUV420_SEMIPLANAR_VU = 2,                        /**< \brief  YYYY... VUVUVU...        */
    ESCL_JPEGENC_YUV422_INTERLEAVED_YUYV = 3,                     /**< \brief  YUYVYUYV...              */
    ESCL_JPEGENC_YUV422_INTERLEAVED_UYVY = 4,                     /**< \brief  UYVYUYVY...              */
    ESCL_JPEGENC_RGB565 = 5,                                      /**< \brief  16-bit RGB 16bpp         */
    ESCL_JPEGENC_BGR565 = 6,                                      /**< \brief  16-bit RGB 16bpp         */
    ESCL_JPEGENC_RGB555 = 7,                                      /**< \brief  15-bit RGB 16bpp         */
    ESCL_JPEGENC_BGR555 = 8,                                      /**< \brief  15-bit RGB 16bpp         */
    ESCL_JPEGENC_RGB444 = 9,                                      /**< \brief  12-bit RGB 16bpp         */
    ESCL_JPEGENC_BGR444 = 10,                                      /**< \brief  12-bit RGB 16bpp         */
    ESCL_JPEGENC_RGB888 = 11,                                      /**< \brief  24-bit RGB 32bpp         */
    ESCL_JPEGENC_BGR888 = 12,                                      /**< \brief  24-bit RGB 32bpp         */
    ESCL_JPEGENC_RGB101010 = 13,                                   /**< \brief  30-bit RGB 32bpp         */
    ESCL_JPEGENC_BGR101010 = 14,                                   /**< \brief  30-bit RGB 32bpp         */
    ESCL_JPEGENC_YUV420_I010 = 15,                            /**< \brief  10-bit YUV 24bpp         */
    ESCL_JPEGENC_YUV420_MS_P010 = 16,                         /**< \brief  10-bit YUVSP 24bpp       */
    ESCL_JPEGENC_YUV420_PLANAR_8BIT_TILE_32_32 = 19,          /**< \brief  8-bit tile 32_32         */
    ESCL_JPEGENC_YUV420_PLANAR_8BIT_TILE_16_16_PACKED_4 = 20, /**< \brief  8-bit tile 16_16 packed 4*/
    ESCL_JPEGENC_YUV420_SEMIPLANAR_8BIT_TILE_4_4 = 21,        /**< \brief  YYYY... UVUVUV...        */
    ESCL_JPEGENC_YUV420_SEMIPLANAR_VU_8BIT_TILE_4_4 = 22,     /**< \brief  YYYY... VUVUVU...        */
    ESCL_JPEGENC_YUV420_PLANAR_10BIT_P010_TILE_4_4 = 23,      /**< \brief  YYYY... UVUV...          */
    ESCL_JPEGENC_YUV420_SEMIPLANAR_101010 = 24,               /**< \brief  YYYY... UVUV...          */
    ESCL_JPEGENC_YUV422_888 = 25,                             /**< \brief  YYYY... UVUVUV...        */
    ESCL_JPEGENC_YUV420_UV_8BIT_TILE_128_2 = 32,              /**< \brief  YYYY... UVUV...          */
    ESCL_JPEGENC_YUV420_8BIT_TILE_8_8 = 35,                   /**< \brief  8-bit 8x8 tile           */
    ESCL_JPEGENC_YUV420_10BIT_TILE_8_8 = 36,                  /**< \brief  10-bit 8x8 tile          */
    ESCL_JPEGENC_YVU420_PLANAR = 37,                          /**< \brief  10-bit 8x8 tile          */
    ESCL_JPEGENC_YUV420_FBC64 = 38,                           /**< \brief  8-bit 64x2 tile          */
    ESCL_JPEGENC_YUV420_UV_10BIT_TILE_128_2 = 39,             /**< \brief  10-bit 128x2 tile        */
    ESCL_JPEGENC_FORMAT_MAX = 40
} escljencJpegEncFrameType;

typedef enum escljencJpegEncPictureRotation
{
    ESCL_JPEGENC_ROTATE_0 = 0,
    ESCL_JPEGENC_ROTATE_90R = 1,
    ESCL_JPEGENC_ROTATE_90L = 2,
    ESCL_JPEGENC_ROTATE_180 = 3
} escljencJpegEncPictureRotation;

typedef enum escljencJpegEncRcMode
{
    ESCL_JPEGENC_SINGLEFRAME = 0,
    ESCL_JPEGENC_CBR = 1,
    ESCL_JPEGENC_VBR = 2
} escljencJpegEncRcMode;

// Aliases data types declarations
typedef struct escljencJInfoReportStat escljencJInfoReportStat;
typedef struct escljencJEncInst escljencJEncInst;
typedef struct escljencJpegEncCfg escljencJpegEncCfg;
typedef struct escljencJpegEncIn escljencJpegEncIn;
typedef struct escljencJpegEncOut escljencJpegEncOut;
typedef struct escljencJpegEncRateCtrl escljencJpegEncRateCtrl;

// Structures/unions data types declarations
struct escljencJInfoReportStat
{
    uint32_t instId;
    uint32_t running;
    uint32_t inQCnt;
    uint32_t inDqCnt;
    uint32_t outQCnt;
    uint32_t outDqCnt;
    uint32_t inWidth;
    uint32_t inHeight;
    uint32_t inBufferNum;
    uint32_t outBufferNum;
    uint64_t beginDateMs;
    uint64_t curDateMs;
    uint32_t AliveDurationMs;
    uint32_t beginMediaTs;
    uint32_t curMediaTs;
    uint32_t codec;
    uint32_t bFramesNum;
    uint32_t profile;
    uint32_t preset;
    uint32_t iFrameInterval;
    uint32_t outTotalBits;
    uint32_t timeBaseNum;
    uint32_t timeBaseDen;
    float inFrameRate;
};

struct escljencJEncInst
{
    uintptr_t inst;
    escljencJInfoReportStat info_report_stat;
};

struct escljencJpegEncCfg
{
    uint32_t inputWidth;         /**< \brief  Number of pixels/line in input image        */
    uint32_t inputHeight;        /**< \brief  Number of lines in input image              */
    uint32_t xOffset;            /**< \brief  Pixels from top-left corner of input image  */
    uint32_t yOffset;            /**< \brief  to top-left corner of encoded image         */
    uint32_t codingWidth;        /**< \brief  Width of encoded image                      */
    uint32_t codingHeight;       /**< \brief  Height of encoded image                     */
    uint32_t restartInterval;    /**< \brief  Restart interval (MCU lines)                */
    uint32_t qLevel;             /**< \brief  Quantization level (0 - 9)                  */
    uint8_t qTableLuma[64];      /**< \brief  Quantization table for luminance [64],
                             * overrides quantization level, zigzag order            */
    uint8_t qTableChroma[64];    /**< \brief  Quantization table for chrominance [64],
                             * overrides quantization level, zigzag order            */
    uint8_t filter[128];         /**< \brief  two 8x8 filter defined for luma and chroma non-roi region*/
    uint32_t enableRoimap;       /**< \brief  Enable roimap*/
    uint32_t nonRoiLevel;        /**< \brief  non_roi filter level (0 - 9)*/
    escljencJpegEncFrameType frameType;             /**< \brief  Input frame YUV / RGB format */
    escljencJpegEncPictureRotation rotation;        /**< \brief  rotation off/-90/+90         */
    uint32_t comLength;          /**< \brief  Length of COM header                         */
    uintptr_t pCom;           /**< \brief  Comment header pointer<todo, var leng comments>        */
    uint32_t inputLineBufDepth;  /**< \brief  input buffer depth in mb lines */
    uint32_t targetBitPerSecond; /**< \brief  targetBitPerSecond*/
    uint32_t frameRateNum;       /**< \brief  The stream time scale, [1..1048575] */
    uint32_t frameRateDenom;
    int32_t fixedQP; /**< \brief  fix qp no RC*/
    escljencJpegEncRcMode rcMode;
    uint32_t exp_of_input_alignment;
    uint32_t xDensity;        /**< \brief  Horizontal pixel density                     */
    uint32_t yDensity;        /**< \brief  Vertical pixel density                       */
    int32_t constChromaEn;
    uint32_t constCb;
    uint32_t constCr;
    uint32_t qpmin;
    uint32_t qpmax;
    int32_t picQpDeltaMin;
    int32_t picQpDeltaMax;
    uint32_t irqTypeMask;
    uint32_t sbi_id_0; /**< \brief  \brief flexa sbi id 0 */
    uint32_t sbi_id_1; /**< \brief  \brief flexa sbi id 1 */
    uint32_t sbi_id_2; /**< \brief  \brief flexa sbi id 2 */
};

struct escljencJpegEncIn
{
    uint32_t frameHeader;       /**< \brief  Enable/disable creation of frame headers     */
    uint64_t busLum;          /**< \brief  Bus address of luminance input   (Y)         */
    uint64_t busCb;           /**< \brief  Bus address of chrominance input (Cb)        */
    uint64_t busCr;           /**< \brief  Bus address of chrominance input (Cr)        */
    uint32_t dec400Enable; /**< \brief 1: bypass 2: enable */
    uint32_t axiFEEnable;  /**< \brief 0: axiFE disable   1:axiFE normal mode enable   2:axiFE bypass  3.security mode*/
    uintptr_t pOutBuf[2];    /**< \brief  Pointer to output buffer              */
    uint64_t busOutBuf[2];      /**< \brief  Bus address of output stream buffer   */
    uint32_t outBufSize[2];     /**< \brief  Size of output buffer (bytes)         */
};

struct escljencJpegEncOut
{
    uint32_t jfifSize;               /**< \brief  Encoded JFIF size (bytes) */
    uint32_t invalidBytesInBuf0Tail; /**< \brief  unused bytes in buf0 tail */
    uint32_t headerSize;
};

struct escljencJpegEncRateCtrl
{
    uint32_t pictureRc;      /**< \brief   Adjust QP between pictures, [0,1] */
    uint32_t qpMin;          /**< \brief   Minimum QP for picture, [0..51] */
    uint32_t qpMax;          /**< \brief   Maximum QP for picture, [0..51] */
    uint32_t bitPerSecond;   /**< \brief  Target bitrate in bits/second, this is
                                  * needed if pictureRc, mbRc, pictureSkip or
                                  * hrd is enabled [10000..60000000] */
    uint32_t bitrateWindow;  /**< \brief   Length for Group of Pictures, indicates
                                  * the distance of two intra pictures,
                                  * including first intra [1..300] */
    uint32_t fixedIntraQp;   /**< \brief   Fixed QP value for all Intra pictures, [0..51]
                                  * 0 = Rate control calculates intra QP. */
    int32_t monitorFrames;
    int32_t picQpDeltaMin;  /**< \brief   minimum pic qp delta */
    uint32_t frameRateNum;   /**< \brief   The stream time scale, [1..1048575] */
    uint32_t frameRateDenom; /**< \brief   Maximum frame rate is frameRateNum/frameRateDenom
                                  * in frames/second. The actual frame rate will be
                                  * defined by timeIncrement of encoded pictures,
                                  * [1..frameRateNum] */
};

static const uint8_t kJencIfId = 4;
static const uint8_t kescljencJpegEncInitId = 1;
static const uint8_t kescljencJpegEncReleaseId = 2;
static const uint8_t kescljencJpegEncSetPictureSizeId = 3;
static const uint8_t kescljencJpegEncBindNumaIdId = 4;
static const uint8_t kescljencJpegEncEncodeId = 5;
static const uint8_t kescljencJpegEncGetRateCtrlId = 6;
static const uint8_t kescljencJpegEncSetRateCtrlId = 7;
static const uint8_t kescljencJpegEncGetPerformanceId = 8;
#endif // ESRPC_TYPE_DEFINITIONS_JENC

#if defined(__cplusplus)
}
#endif

#endif // _JENC_JENCIF_COMMON_H_
