// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESCL__MEMORY_TYPE_H__
#define __ESCL__MEMORY_TYPE_H__

// Add extra memory headers here.
#include "escl_type.h"
#include "escl_status.h"

typedef enum {
    UID_PRIVATE = 0,
    UID_COMMON = 1,
    UID_VI = 2,
    UID_VO = 3,
    UID_VPS = 4,
    UID_VENC = 5,
    UID_VDEC = 6,
    UID_HAE = 7,
    UID_USER = 8,
    UID_BUTT = 9,
    UID_MAX = 10
} ESCLMemHwUid;

typedef enum {
    ESCL_CACHE_MODE_NOCACHE = 0,
    ESCL_CACHE_MODE_CACHED = 1,
    ESCL_CACHE_MODE_MAX = 2,
} ESCLMemCacheMode;

typedef struct {
    int8_t strZone[32];
    uint64_t free;
    uint64_t total;
} VbBlock;

typedef struct {
    uint32_t zoneCount;
    VbBlock mems[4];
} ESCLMemInfo;

#define ESCL_MEMORY_ERROR_BASE ESCL_DEF_ERR(ESCL_ID_MEMORY, ESCL_ERR_LEVEL_ERROR, 0)
#define ESCL_MEMORY_ERROR_NUM(X) (ESCL_MEMORY_ERROR_BASE + X)

#define ESCL_MEMORY_ERROR_LOGIC_ERROR ESCL_MEMORY_ERROR_NUM(0)
#define ESCL_MEMORY_ERROR_DST_ADDR ESCL_MEMORY_ERROR_NUM(1)
#define ESCL_MEMORY_ERROR_SRC_ADDR ESCL_MEMORY_ERROR_NUM(2)
#define ESCL_MEMORY_ERROR_SRC_OUT_OF_RANGE ESCL_MEMORY_ERROR_NUM(3)
#define ESCL_MEMORY_ERROR_DST_OUT_OF_RANGE ESCL_MEMORY_ERROR_NUM(4)
#define ESCL_MEMORY_ERROR_NO_MEM ESCL_MEMORY_ERROR_NUM(5)
#define ESCL_MEMORY_ERROR_NOT_PERMIT ESCL_MEMORY_ERROR_NUM(6)
#define ESCL_MEMORY_ERROR_OPERATION_FAIL ESCL_MEMORY_ERROR_NUM(7)
#define ESCL_MEMORY_ERROR_INVALID_PARAM ESCL_MEMORY_ERROR_NUM(12)

#endif  //__ESCL__MEMORY_TYPE_H__