/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_TDE_TDEIF_COMMON_H_)
#define _TDE_TDEIF_COMMON_H_
#include "tde_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#if !defined(ESRPC_TYPE_DEFINITIONS_TDE)
#define ESRPC_TYPE_DEFINITIONS_TDE

// Enumerators data types declarations
typedef enum escltdeStatus
{
    ESCL_TDE_STATUS_SUCCESS = 0,
    ESCL_TDE_STATUS_FAILURE = -1,
    ESCL_TDE_STATUS_INVALID_PARAM = -2,
    ESCL_TDE_STATUS_OUT_OF_MEMORY = -3,
    ESCL_TDE_STATUS_NOT_SUPPORTED = -4,
    ESCL_TDE_STATUS_NOT_PERM = -5
} escltdeStatus;

typedef enum escltdeFrameFormat
{
    ESCL_TDE_FMT_NV12 = 0,
    ESCL_TDE_FMT_NV21 = 1,
    ESCL_TDE_FMT_I420 = 2,
    ESCL_TDE_FMT_YV12 = 3,
    ESCL_TDE_FMT_YUY2 = 4,
    ESCL_TDE_FMT_YVY2 = 5,
    ESCL_TDE_FMT_UYVY = 6,
    ESCL_TDE_FMT_VYUY = 7,
    ESCL_TDE_FMT_NV16 = 8,
    ESCL_TDE_FMT_NV61 = 9,
    ESCL_TDE_FMT_I010 = 10,
    ESCL_TDE_FMT_I010_HI = 11,
    ESCL_TDE_FMT_P010 = 12,
    ESCL_TDE_FMT_P010_LO = 13,
    ESCL_TDE_FMT_GRAY8 = 14,
    ESCL_TDE_FMT_YUV444P = 15,
    ESCL_TDE_FMT_YUV422SP_10BIT = 16,
    ESCL_TDE_FMT_YUV_BUTT = 17,
    ESCL_TDE_FMT_X4R4G4B4 = 65536,
    ESCL_TDE_FMT_A4R4G4B4 = 65537,
    ESCL_TDE_FMT_R4G4B4X4 = 65538,
    ESCL_TDE_FMT_R4G4B4A4 = 65539,
    ESCL_TDE_FMT_X1R5G5B5 = 65540,
    ESCL_TDE_FMT_A1R5G5B5 = 65541,
    ESCL_TDE_FMT_R5G5B5X1 = 65542,
    ESCL_TDE_FMT_R5G5B5A1 = 65543,
    ESCL_TDE_FMT_R5G6B5 = 65544,
    ESCL_TDE_FMT_R8G8B8 = 65545,
    ESCL_TDE_FMT_X8R8G8B8 = 65546,
    ESCL_TDE_FMT_R8G8B8X8 = 65547,
    ESCL_TDE_FMT_A8R8G8B8 = 65548,
    ESCL_TDE_FMT_R8G8B8A8 = 65549,
    ESCL_TDE_FMT_A2R10G10B10 = 65550,
    ESCL_TDE_FMT_R10G10B10A2 = 65551,
    ESCL_TDE_FMT_R4G4B4 = 65552,
    ESCL_TDE_FMT_R5G5B5 = 65553,
    ESCL_TDE_FMT_R10G10B10 = 65554,
    ESCL_TDE_FMT_X4B4G4R4 = 65555,
    ESCL_TDE_FMT_B4G4R4X4 = 65556,
    ESCL_TDE_FMT_A4B4G4R4 = 65557,
    ESCL_TDE_FMT_B4G4R4A4 = 65558,
    ESCL_TDE_FMT_X1B5G5R5 = 65559,
    ESCL_TDE_FMT_B5G5R5X1 = 65560,
    ESCL_TDE_FMT_A1B5G5R5 = 65561,
    ESCL_TDE_FMT_B5G5R5A1 = 65562,
    ESCL_TDE_FMT_B5G6R5 = 65563,
    ESCL_TDE_FMT_B8G8R8 = 65564,
    ESCL_TDE_FMT_X8B8G8R8 = 65565,
    ESCL_TDE_FMT_B8G8R8X8 = 65566,
    ESCL_TDE_FMT_A8B8G8R8 = 65567,
    ESCL_TDE_FMT_B8G8R8A8 = 65568,
    ESCL_TDE_FMT_A2B10G10R10 = 65569,
    ESCL_TDE_FMT_B10G10R10A2 = 65570,
    ESCL_TDE_FMT_B4G4R4 = 65571,
    ESCL_TDE_FMT_B5G5R5 = 65572,
    ESCL_TDE_FMT_B10G10R10 = 65573,
    ESCL_TDE_FMT_RGB_BUTT = 65574,
    ESCL_TDE_FMT_BUTT = 65575
} escltdeFrameFormat;

typedef enum escltdeFuncType
{
    ESCL_TDE_FUNC_TYPE_CSC = 0,
    ESCL_TDE_FUNC_TYPE_CROP = 1,
    ESCL_TDE_FUNC_TYPE_RESIZE = 2,
    ESCL_TDE_FUNC_TYPE_ROTATION_90_270 = 3,
    ESCL_TDE_FUNC_TYPE_ROTATION_180 = 4,
    ESCL_TDE_FUNC_TYPE_FLIP_MIRROR = 5,
    ESCL_TDE_FUNC_TYPE_FILL = 6,
    ESCL_TDE_FUNC_TYPE_MULTISOURCE = 7,
    ESCL_TDE_FUNC_TYPE_LOGO_YUV = 8,
    ESCL_TDE_FUNC_TYPE_LOGO_RGB = 9,
    ESCL_TDE_FUNC_TYPE_BUTT = 10
} escltdeFuncType;

typedef enum escltdeRotation
{
    ESCL_TDE_ROTATION_0 = 0,
    ESCL_TDE_ROTATION_FLIP_H = 1,
    ESCL_TDE_ROTATION_FLIP_V = 2,
    ESCL_TDE_ROTATION_180 = 3,
    ESCL_TDE_ROTATION_90 = 4,
    ESCL_TDE_ROTATION_270 = 7,
    ESCL_TDE_ROTATION_BUTT = 8
} escltdeRotation;

typedef enum escltdeMoudleMode
{
    ESCL_TDE_MODULE_COMPLEX = 0,
    ESCL_TDE_MODULE_STACK = 1,
    ESCL_TDE_MODULE_BUTT = 2
} escltdeMoudleMode;

// Aliases data types declarations
typedef struct list_s8_1_t list_s8_1_t;
typedef struct escltdeRealTimeInfo escltdeRealTimeInfo;
typedef struct escltdeBaseInfo escltdeBaseInfo;
typedef struct escltdeBufInfo escltdeBufInfo;
typedef struct list_escltdeBufInfo_1_t list_escltdeBufInfo_1_t;
typedef struct escltdeRect escltdeRect;
typedef struct list_escltdeRect_1_t list_escltdeRect_1_t;
typedef struct escltdeRectInfo escltdeRectInfo;
typedef struct list_escltdeRectInfo_1_t list_escltdeRectInfo_1_t;
typedef struct escltdeInitInfo escltdeInitInfo;
typedef list_s8_1_t charList;
typedef list_escltdeRectInfo_1_t escltdeRectInfoList;
typedef list_escltdeRect_1_t escltdeRectList;
typedef list_escltdeBufInfo_1_t escltdeBufInfoList;

// Structures/unions data types declarations
struct list_s8_1_t
{
    int8_t * elements;
    uint32_t elementsCount;
};

struct escltdeRealTimeInfo
{
    uint64_t curDateMs;
    uint64_t AliveDurationMs;
    uint32_t curMediaTs;
    uint32_t inBufferNum;
    uint32_t outBufferNum;
    uint32_t failBufferNum;
    uint32_t snapshotCnt;
    uint32_t snapshotTotalBits;
    float inFrameRate;
    float outFrameRate;
};

struct escltdeBaseInfo
{
    int32_t modeId;
    uint32_t running;
    uint32_t inSCnt;
    uint32_t inWidth;
    uint32_t inHeight;
    uint32_t outWidth;
    uint32_t outHeight;
    uint64_t beginDateMs;
    uint32_t timeBaseNum;
    uint32_t timeBaseDen;
    uint32_t beginMediaTs;
    escltdeFuncType funcTypes;
    escltdeRealTimeInfo rtInfo;
};

struct escltdeBufInfo
{
    uint64_t vir_addr[3];
    uint64_t iova_addr[3];
    uint64_t fd;
    int32_t buffer_size;
    uint32_t width;
    uint32_t height;
    escltdeFrameFormat format;
    uint32_t wstride;
    uint32_t hstride;
    uint32_t stride[3];
    uint32_t offset[3];
    uint32_t planeCnt;
    int32_t rotation;
    int32_t global_alpha;
    uintptr_t handle;
    charList reportInfo;
};

struct list_escltdeBufInfo_1_t
{
    escltdeBufInfo * elements;
    uint32_t elementsCount;
};

struct escltdeRect
{
    int32_t x;
    int32_t y;
    int32_t width;
    int32_t height;
};

struct list_escltdeRect_1_t
{
    escltdeRect * elements;
    uint32_t elementsCount;
};

struct escltdeRectInfo
{
    escltdeRect rect;
    uint32_t color;
    int32_t thickness;
};

struct list_escltdeRectInfo_1_t
{
    escltdeRectInfo * elements;
    uint32_t elementsCount;
};

struct escltdeInitInfo
{
    uint32_t nId;
    escltdeMoudleMode moduleId;
};

static const uint8_t kTdeIfId = 2;
static const uint8_t kescltdeCreateHandleId = 1;
static const uint8_t kescltdeDestoryHandleId = 2;
static const uint8_t kescltdeFillId = 3;
static const uint8_t kescltdeComplexId = 4;
static const uint8_t kescltdeMultisourceId = 5;
static const uint8_t kescltdeLogoId = 6;
#endif // ESRPC_TYPE_DEFINITIONS_TDE

#if defined(__cplusplus)
}
#endif

#endif // _TDE_TDEIF_COMMON_H_
