/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_VDEC_VDECIF_COMMON_H_)
#define _VDEC_VDECIF_COMMON_H_
#include "vdec_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#if !defined(ESRPC_TYPE_DEFINITIONS_VDEC)
#define ESRPC_TYPE_DEFINITIONS_VDEC

// Enumerators data types declarations
/*! end of decsei.h*/
/*! decapicommon.h*/
/**
 * macro definition
 */
enum
{
    ESCL_MAX_ASIC_CORES = 4
};

typedef enum esclvdecDecErrorHandling
{
    ESCL_DEC_EC_PIC_COPY_REF = 1,
    ESCL_DEC_EC_PIC_PARTIAL = 2,
    ESCL_DEC_EC_PIC_PARTIAL_IGNORE = 4,
    ESCL_DEC_EC_PIC_ALL_IGNORE = 8,
    ESCL_DEC_EC_REF_REPLACE = 256,
    ESCL_DEC_EC_REF_NEXT_IDR = 512,
    ESCL_DEC_EC_REF_NEXT_I = 1024,
    ESCL_DEC_EC_OUT_ALL = 65536,
    ESCL_DEC_EC_OUT_NO_ERROR = 131072,
    ESCL_DEC_EC_OUT_FIRST_FIELD_OK = 262144,
    ESCL_DEC_EC_PICTURE_FREEZE = 1,
    ESCL_DEC_EC_VIDEO_FREEZE = 513,
    ESCL_DEC_EC_PARTIAL_FREEZE = 2,
    ESCL_DEC_EC_PARTIAL_IGNORE = 4,
    ESCL_DEC_EC_FAST_FREEZE = 132104
} esclvdecDecErrorHandling;

typedef enum esclvdecDecDpbMode
{
    ESCL_DEC_DPB_FRAME = 0,
    ESCL_DEC_DPB_INTERLACED_FIELD = 1
} esclvdecDecDpbMode;

typedef enum esclvdecDecDecoderMode
{
    ESCL_DEC_NORMAL = 0,
    ESCL_DEC_LOW_LATENCY = 1,
    ESCL_DEC_SECURITY = 2,
    ESCL_DEC_INTRA_ONLY = 8
} esclvdecDecDecoderMode;

typedef enum esclvdecDecPictureFormat
{
    ESCL_DEC_OUT_FRM_TILED_4X4 = 0,
    ESCL_DEC_OUT_FRM_TILED_8X4 = 1,
    ESCL_DEC_OUT_FRM_RASTER_SCAN = 2,
    ESCL_DEC_OUT_FRM_PLANAR_420 = 3,
    ESCL_DEC_OUT_FRM_TILED_8X8 = 4,
    ESCL_DEC_OUT_FRM_TILED_16X16 = 5,
    ESCL_DEC_OUT_FRM_MONOCHROME = 6,
    ESCL_DEC_OUT_FRM_RFC = 7,
    ESCL_DEC_OUT_FRM_DEC400 = 8,
    ESCL_DEC_OUT_FRM_DEC400_TILE = 9,
    ESCL_DEC_OUT_FRM_DEC400_P = 10,
    ESCL_DEC_OUT_FRM_YUV420TILE = 11,
    ESCL_DEC_OUT_FRM_YUV420TILE_PACK10 = 12,
    ESCL_DEC_OUT_FRM_YUV420TILE_P010 = 13,
    ESCL_DEC_OUT_FRM_YUV420TILE_I010 = 14,
    ESCL_DEC_OUT_FRM_YUV420TILE_L010 = 15,
    ESCL_DEC_OUT_FRM_YUV420TILE_P012 = 16,
    ESCL_DEC_OUT_FRM_YUV420TILE_I012 = 17,
    ESCL_DEC_OUT_FRM_YUV420TILE_128X2 = 18,
    ESCL_DEC_OUT_FRM_YUV420TILE_128X2_P010 = 19,
    ESCL_DEC_OUT_FRM_YUV420SP = 20,
    ESCL_DEC_OUT_FRM_YUV420SP_PACK10 = 21,
    ESCL_DEC_OUT_FRM_YUV420SP_P010 = 22,
    ESCL_DEC_OUT_FRM_YUV420SP_I010 = 23,
    ESCL_DEC_OUT_FRM_YUV420SP_L010 = 24,
    ESCL_DEC_OUT_FRM_YUV420SP_P012 = 25,
    ESCL_DEC_OUT_FRM_YUV420SP_I012 = 26,
    ESCL_DEC_OUT_FRM_YUV420SP_1010 = 27,
    ESCL_DEC_OUT_FRM_YUV420P = 28,
    ESCL_DEC_OUT_FRM_YUV420P_PACK10 = 29,
    ESCL_DEC_OUT_FRM_YUV420P_P010 = 30,
    ESCL_DEC_OUT_FRM_YUV420P_I010 = 31,
    ESCL_DEC_OUT_FRM_YUV420P_L010 = 32,
    ESCL_DEC_OUT_FRM_YUV420P_P012 = 33,
    ESCL_DEC_OUT_FRM_YUV420P_I012 = 34,
    ESCL_DEC_OUT_FRM_YUV420P_1010 = 35,
    ESCL_DEC_OUT_FRM_YUV400TILE = 36,
    ESCL_DEC_OUT_FRM_YUV400TILE_PACK10 = 37,
    ESCL_DEC_OUT_FRM_YUV400TILE_P010 = 38,
    ESCL_DEC_OUT_FRM_YUV400TILE_I010 = 39,
    ESCL_DEC_OUT_FRM_YUV400TILE_L010 = 40,
    ESCL_DEC_OUT_FRM_YUV400TILE_P012 = 41,
    ESCL_DEC_OUT_FRM_YUV400TILE_I012 = 42,
    ESCL_DEC_OUT_FRM_YUV400 = 43,
    ESCL_DEC_OUT_FRM_YUV400_PACK10 = 44,
    ESCL_DEC_OUT_FRM_YUV400_P010 = 45,
    ESCL_DEC_OUT_FRM_YUV400_I010 = 46,
    ESCL_DEC_OUT_FRM_YUV400_L010 = 47,
    ESCL_DEC_OUT_FRM_YUV400_P012 = 48,
    ESCL_DEC_OUT_FRM_YUV400_I012 = 49,
    ESCL_DEC_OUT_FRM_YUV400TILE_128X2 = 50,
    ESCL_DEC_OUT_FRM_YUV400TILE_128X2_P010 = 51,
    ESCL_DEC_OUT_FRM_YUV422SP = 52,
    ESCL_DEC_OUT_FRM_YUV422SP_1010 = 53,
    ESCL_DEC_OUT_FRM_YUV440 = 54,
    ESCL_DEC_OUT_FRM_YUV411SP = 55,
    ESCL_DEC_OUT_FRM_YUV444SP = 56,
    ESCL_DEC_OUT_FRM_NV21TILE = 57,
    ESCL_DEC_OUT_FRM_NV21TILE_PACK10 = 58,
    ESCL_DEC_OUT_FRM_NV21TILE_P010 = 59,
    ESCL_DEC_OUT_FRM_NV21TILE_I010 = 60,
    ESCL_DEC_OUT_FRM_NV21TILE_L010 = 61,
    ESCL_DEC_OUT_FRM_NV21TILE_P012 = 62,
    ESCL_DEC_OUT_FRM_NV21TILE_I012 = 63,
    ESCL_DEC_OUT_FRM_NV21SP = 64,
    ESCL_DEC_OUT_FRM_NV21SP_PACK10 = 65,
    ESCL_DEC_OUT_FRM_NV21SP_P010 = 66,
    ESCL_DEC_OUT_FRM_NV21SP_I010 = 67,
    ESCL_DEC_OUT_FRM_NV21SP_L010 = 68,
    ESCL_DEC_OUT_FRM_NV21SP_P012 = 69,
    ESCL_DEC_OUT_FRM_NV21SP_I012 = 70,
    ESCL_DEC_OUT_FRM_NV21SP_1010 = 71,
    ESCL_DEC_OUT_FRM_NV21P = 72,
    ESCL_DEC_OUT_FRM_NV21P_PACK10 = 73,
    ESCL_DEC_OUT_FRM_NV21P_P010 = 74,
    ESCL_DEC_OUT_FRM_NV21P_I010 = 75,
    ESCL_DEC_OUT_FRM_NV21P_L010 = 76,
    ESCL_DEC_OUT_FRM_NV21P_P012 = 77,
    ESCL_DEC_OUT_FRM_NV21P_I012 = 78,
    ESCL_DEC_OUT_FRM_NV21P_1010 = 79,
    ESCL_DEC_OUT_FRM_RGB888_P = 80,
    ESCL_DEC_OUT_FRM_BGR888 = 81,
    ESCL_DEC_OUT_FRM_BGR888_P = 82,
    ESCL_DEC_OUT_FRM_RGB888 = 83,
    ESCL_DEC_OUT_FRM_R16G16B16_P = 84,
    ESCL_DEC_OUT_FRM_B16G16R16 = 85,
    ESCL_DEC_OUT_FRM_B16G16R16_P = 86,
    ESCL_DEC_OUT_FRM_R16G16B16 = 87,
    ESCL_DEC_OUT_FRM_ABGR888 = 88,
    ESCL_DEC_OUT_FRM_ARGB888 = 89,
    ESCL_DEC_OUT_FRM_A2B10G10R10 = 90,
    ESCL_DEC_OUT_FRM_A2R10G10B10 = 91,
    ESCL_DEC_OUT_FRM_XBGR888 = 92,
    ESCL_DEC_OUT_FRM_XRGB888 = 93
} esclvdecDecPictureFormat;

typedef enum esclvdecDecPicAlignment
{
    ESCL_DEC_ALIGN_1B = 0, /**< Alignment: 1 byte */
    ESCL_DEC_ALIGN_8B = 3, /**< Alignment: 8 bytes */
    ESCL_DEC_ALIGN_16B = 4,    /**< Alignment: 16 bytes */
    ESCL_DEC_ALIGN_32B = 5,    /**< Alignment: 32 bytes */
    ESCL_DEC_ALIGN_64B = 6,    /**< Alignment: 64 bytes */
    ESCL_DEC_ALIGN_128B = 7,   /**< Alignment: 128 bytes */
    ESCL_DEC_ALIGN_256B = 8,   /**< Alignment: 256 bytes */
    ESCL_DEC_ALIGN_512B = 9,   /**< Alignment: 512 bytes */
    ESCL_DEC_ALIGN_1024B = 10,  /**< Alignment: 1024 bytes */
    ESCL_DEC_ALIGN_2048B = 11  /**< Alignment: 2048 bytes */
} esclvdecDecPicAlignment;

/*! end of decapicommon.h*/
/*! dectypes.h*/
/**
 * pre definition
*/
enum
{
    ESCL_DEC_MAX_OUT_COUNT = 6,
    ESCL_DEC_MAX_PPU_COUNT = 6
};

enum
{
    ESCL_BT601 = 0,
    ESCL_BT601_L = 1,
    ESCL_BT709 = 2,
    ESCL_BT709_L = 3,
    ESCL_BT2020 = 4,
    ESCL_BT2020_L = 5
};

typedef enum esclvdecDecRet
{
    /** Success */
    ESCL_DEC_OK = 0, /**<\hideinitializer */
    /** Stream processed */
    ESCL_DEC_STRM_PROCESSED = 1, /**<\hideinitializer */
    /** Picture available for output */
    ESCL_DEC_PIC_RDY = 2, /**<\hideinitializer */
    /** Picture decoded */
    ESCL_DEC_PIC_DECODED = 3, /**<\hideinitializer */
    /** New stream headers decoded */
    ESCL_DEC_HDRS_RDY = 4, /**<\hideinitializer */
    /** New stream headers decoded, data partitioned stream(mpeg4 specific) */
    ESCL_DEC_DP_HDRS_RDY = 5, /**<\hideinitializer */
    /** Advanced coding tools detected in stream */
    ESCL_DEC_ADVANCED_TOOLS = 6, /**<\hideinitializer */
    /** A slice was decoded */
    ESCL_DEC_SLICE_RDY = 7, /**<\hideinitializer */
    /** Output pictures must be retrieved before continuing decode */
    ESCL_DEC_PENDING_FLUSH = 8, /**<\hideinitializer */
    /** Skipped decoding non-reference picture */
    ESCL_DEC_NONREF_PIC_SKIPPED = 9, /**<\hideinitializer */
    /** End-of-stream state set in the decoder */
    ESCL_DEC_END_OF_STREAM = 10,         /**<\hideinitializer */
    /** End of video sequence */
    ESCL_DEC_END_OF_SEQ = 11,         /**<\hideinitializer */
    /** Waiting for external buffers allocated. */
    ESCL_DEC_WAITING_FOR_BUFFER = 12,    /**<\hideinitializer */
    /** Scan Processed(pjpeg specific) */
    ESCL_DEC_SCAN_PROCESSED = 13,     /**<\hideinitializer */
    /** Decoder is aborted */
    ESCL_DEC_ABORTED = 14,              /**<\hideinitializer */
    /** All pics have been flushed */
    ESCL_DEC_FLUSHED = 15,              /**<\hideinitializer */
    /** The input buffer has consumed, but the decoding has not been completed */
    ESCL_DEC_BUF_EMPTY = 16,              /**<\hideinitializer */
    /** Syntax element error found in stream */
    ESCL_DEC_STREAM_ERROR_DEDECTED = 17,  /**<\hideinitializer */
    /** Resolution changed in multiresolution video */
    ESCL_DEC_RESOLUTION_CHANGED = 18,    /**<\hideinitializer */
    /** video Object Sequence end marker dedected */
    ESCL_DEC_VOS_END = 19,    /**<\hideinitializer */
    /** Invalid parameter was used */
    ESCL_DEC_PARAM_ERROR = -1,          /**<\hideinitializer */
    ESCL_DEC_STRM_ERROR = -2,           /**<\hideinitializer */
    /** The decoder has not been initialized */
    ESCL_DEC_NOT_INITIALIZED = -3,      /**<\hideinitializer */
    /** Memory allocation failed */
    ESCL_DEC_MEMFAIL = -4,              /**<\hideinitializer */
    /** Initialization failed */
    ESCL_DEC_INITFAIL = -5,             /**<\hideinitializer */
    /** Video sequence information is not available because stream headers have not been decoded */
    ESCL_DEC_HDRS_NOT_RDY = -6,         /**<\hideinitializer */
    /** Video sequence frame size or tools not supported */
    ESCL_DEC_STREAM_NOT_SUPPORTED = -8, /**<\hideinitializer */
    /** External buffer rejected. (Too much than requested) */
    ESCL_DEC_EXT_BUFFER_REJECTED = -9,    /**<\hideinitializer */
    /** Invalid pp parameter was used */
    ESCL_DEC_INFOPARAM_ERROR = -10,    /**<\hideinitializer */
    /** ERROR */
    ESCL_DEC_ERROR = -11,              /**<\hideinitializer */
    /** Unsupported */
    ESCL_DEC_UNSUPPORTED = -12,        /**<\hideinitializer */
    /** Invalid stream length used */
    ESCL_DEC_INVALID_STREAM_LENGTH = -13, /**<\hideinitializer */
    /** Invalid input buffer size used */
    ESCL_DEC_INVALID_INPUT_BUFFER_SIZE = -14, /**<\hideinitializer */
    /** Needs to increase input buffer */
    ESCL_DEC_INCREASE_INPUT_BUFFER = -15, /**<\hideinitializer */
    /** Slice mode not supported for this operation type */
    ESCL_DEC_SLICE_MODE_UNSUPPORTED = -16, /**<\hideinitializer */
    /** Supplied metadata is in wrong format */
    ESCL_DEC_METADATA_FAIL = -17,     /**<\hideinitializer */
    /** The decoder has no available buffer */
    ESCL_DEC_NO_DECODING_BUFFER = -99,  /**<\hideinitializer */
    /** Driver could not reserve decoder hardware */
    ESCL_DEC_HW_RESERVED = -254,        /**<\hideinitializer */
    /** Hardware timeout occurred */
    ESCL_DEC_HW_TIMEOUT = -255,         /**<\hideinitializer */
    /** Hardware received error status from system bus */
    ESCL_DEC_HW_BUS_ERROR = -256,       /**<\hideinitializer */
    /** Hardware encountered an unrecoverable system error */
    ESCL_DEC_SYSTEM_ERROR = -257,       /**<\hideinitializer */
    /** Decoder wrapper encountered an error */
    ESCL_DEC_DWL_ERROR = -258,          /**<\hideinitializer */
    /** Hardware encountered an fatal system error */
    ESCL_DEC_FATAL_SYSTEM_ERROR = -259,       /**<\hideinitializer */
    /** Evaluation limit exceeded */
    ESCL_DEC_EVALUATION_LIMIT_EXCEEDED = -999, /**<\hideinitializer */
    /** Video format not supported */
    ESCL_DEC_FORMAT_NOT_SUPPORTED = -1000 /**<\hideinitializer */
} esclvdecDecRet;

typedef enum esclvdecDecCodec
{
    ESCL_DEC_MPEG4 = 0,
    ESCL_DEC_MPEG2 = 1,
    ESCL_DEC_VP6 = 2,
    ESCL_DEC_VP8 = 3,
    ESCL_DEC_VP9 = 4,
    ESCL_DEC_HEVC = 5,
    ESCL_DEC_H264_H10P = 6,
    ESCL_DEC_H264 = 7,
    ESCL_DEC_AVS = 8,
    ESCL_DEC_AVS2 = 9,
    ESCL_DEC_AV1 = 10,
    ESCL_DEC_JPEG = 11,
    ESCL_DEC_VC1 = 12,
    ESCL_DEC_RV = 13,
    ESCL_DEC_FMT_MAX = 14
} esclvdecDecCodec;

typedef enum esclvdecDecErrorConcealment
{
    ESCL_DEC_PICTURE_FREEZE = 0,
    ESCL_DEC_INTRA_FREEZE = 1
} esclvdecDecErrorConcealment;

typedef enum esclvdecDecInputFormat
{
    ESCL_DEC_INPUT_VP7 = 1,
    ESCL_DEC_INPUT_VP8 = 2,
    ESCL_DEC_INPUT_WEBP = 3,
    ESCL_DEC_INPUT_MPEG4 = 4,
    ESCL_DEC_INPUT_SORENSON = 5,
    ESCL_DEC_INPUT_CUSTOM_1 = 6
} esclvdecDecInputFormat;

typedef enum esclvdecDecSkipFrameMode
{
    ESCL_DEC_SKIP_NON_REF_RECON = 0,
    ESCL_DEC_SKIP_NON_REF = 1,
    ESCL_DEC_SKIP_NONE = 2
} esclvdecDecSkipFrameMode;

typedef enum esclvdecDecVideoRange
{
    ESCL_DEC_VIDEO_RANGE_NORMAL = 0,
    ESCL_DEC_VIDEO_RANGE_FULL = 1,
    ESCL_DEC_VIDEO_STUDIO_SWING = 0,
    ESCL_DEC_VIDEO_FULL_SWING = 1
} esclvdecDecVideoRange;

enum
{
    ESCL_DWL_CLIENT_TYPE_H264_DEC = 1,
    ESCL_DWL_CLIENT_TYPE_MPEG4_DEC = 2,
    ESCL_DWL_CLIENT_TYPE_JPEG_DEC = 3,
    ESCL_DWL_CLIENT_TYPE_PP = 4,
    ESCL_DWL_CLIENT_TYPE_VC1_DEC = 5,
    ESCL_DWL_CLIENT_TYPE_MPEG2_DEC = 6,
    ESCL_DWL_CLIENT_TYPE_VP6_DEC = 7,
    ESCL_DWL_CLIENT_TYPE_AVS_DEC = 8,
    ESCL_DWL_CLIENT_TYPE_RV_DEC = 9,
    ESCL_DWL_CLIENT_TYPE_VP8_DEC = 10,
    ESCL_DWL_CLIENT_TYPE_VP9_DEC = 11,
    ESCL_DWL_CLIENT_TYPE_HEVC_DEC = 12,
    ESCL_DWL_CLIENT_TYPE_ST_PP = 14,
    ESCL_DWL_CLIENT_TYPE_H264_MAIN10 = 15,
    ESCL_DWL_CLIENT_TYPE_AVS2_DEC = 16,
    ESCL_DWL_CLIENT_TYPE_AV1_DEC = 17,
    ESCL_DWL_CLIENT_TYPE_MAX = 18
};

enum
{
    ESCL_DWL_MEM_TYPE_CPU = 0,
    ESCL_DWL_MEM_TYPE_SLICE = 1,
    ESCL_DWL_MEM_TYPE_DPB = 2,
    ESCL_DWL_MEM_TYPE_VPU_WORKING = 3,
    ESCL_DWL_MEM_TYPE_VPU_WORKING_SPECIAL = 4,
    ESCL_DWL_MEM_TYPE_VPU_ONLY = 5,
    ESCL_DWL_MEM_TYPE_VPU_CPU = 6,
    ESCL_DWL_MEM_TYPE_DMA_DEVICE_ONLY = 256,
    ESCL_DWL_MEM_TYPE_DMA_HOST_TO_DEVICE = 512,
    ESCL_DWL_MEM_TYPE_DMA_DEVICE_TO_HOST = 768,
    ESCL_DWL_MEM_TYPE_DMA_HOST_AND_DEVICE = 1024
};

// Aliases data types declarations
typedef struct esclvdecInfoReportStat esclvdecInfoReportStat;
typedef struct esclvdecDecInst esclvdecDecInst;
typedef struct esclvdecHevcMasterDisParameters esclvdecHevcMasterDisParameters;
typedef struct esclvdecHevcLightLevelParameters esclvdecHevcLightLevelParameters;
typedef struct esclvdecHevcSEIParameters esclvdecHevcSEIParameters;
typedef struct esclvdecH264MasterDisParameters esclvdecH264MasterDisParameters;
typedef struct esclvdecH264LightLevelParameters esclvdecH264LightLevelParameters;
typedef struct esclvdecH264SEIParameters esclvdecH264SEIParameters;
typedef struct esclvdecDecSEIParameters esclvdecDecSEIParameters;
typedef struct esclvdecDecHwConfig esclvdecDecHwConfig;
typedef struct esclvdecDecApiVersion esclvdecDecApiVersion;
typedef struct esclvdecDecSwHwBuild esclvdecDecSwHwBuild;
typedef struct esclvdecCrop esclvdecCrop;
typedef struct esclvdecCrop2 esclvdecCrop2;
typedef struct esclvdecScale esclvdecScale;
typedef struct esclvdecPpUnitConfig esclvdecPpUnitConfig;
typedef struct esclvdecDWLInitParam esclvdecDWLInitParam;
typedef struct esclvdecDWLLinearMemAllocInfo esclvdecDWLLinearMemAllocInfo;
typedef struct esclvdecDWLLinearMem esclvdecDWLLinearMem;
typedef struct esclvdecDecParams esclvdecDecParams;
typedef struct esclvdecDecCropParams esclvdecDecCropParams;
typedef struct esclvdecDecOutput esclvdecDecOutput;
typedef struct esclvdecDecInputParameters esclvdecDecInputParameters;
typedef struct esclvdecDecMCConfig esclvdecDecMCConfig;
typedef struct esclvdecDecInitConfig esclvdecDecInitConfig;
typedef struct esclvdecDecConfig esclvdecDecConfig;
typedef struct esclvdecDecSequenceInfo esclvdecDecSequenceInfo;
typedef struct esclvdecDecPictureInfo esclvdecDecPictureInfo;
typedef struct esclvdecDecPicture esclvdecDecPicture;
typedef struct esclvdecDecPicturePpu esclvdecDecPicturePpu;
typedef struct esclvdecDecBufferInfo esclvdecDecBufferInfo;

// Structures/unions data types declarations
struct esclvdecInfoReportStat
{
    uint32_t instId;
    uint32_t running;
    uint32_t inQCnt;
    uint32_t inDqCnt;
    uint32_t outQCnt;
    uint32_t outDqCnt;
    uint32_t inWidth;
    uint32_t inHeight;
    uint32_t inBufferNum;
    uint32_t outBufferNum;
    uint64_t beginDateMs;
    uint64_t curDateMs;
    uint32_t AliveDurationMs;
    uint32_t beginMediaTs;
    uint32_t curMediaTs;
    uint32_t codec;
    uint32_t inTotalBits;
    uint32_t timeBaseNum;
    uint32_t timeBaseDen;
    float inFrameRate;
};

struct esclvdecDecInst
{
    uintptr_t inst;
    esclvdecInfoReportStat info_report_stat;
};

/*! decsei.h*/
/**
 * \struct esclvdecHevcMasterDisParameters
 * \brief structure to store Mastering Display Colour Volume SEI parameters
 * \ingroup hevc_group
 */
struct esclvdecHevcMasterDisParameters
{
    uint16_t display_primaries_x[3];
    uint16_t display_primaries_y[3];
    uint16_t white_point_x;
    uint16_t white_point_y;
    uint32_t max_display_mastering_luminance;
    uint32_t min_display_mastering_luminance;
};

/**
 * \struct esclvdecHevcLightLevelParameters
 * \brief structure to store Content Light Level Info SEI parameters
 * \ingroup hevc_group
 */
struct esclvdecHevcLightLevelParameters
{
    uint16_t max_content_light_level;
    uint16_t max_pic_average_light_level;
};

struct esclvdecHevcSEIParameters
{
    esclvdecHevcMasterDisParameters masterdis_param;
    esclvdecHevcLightLevelParameters light_param;
};

/**
 * \struct esclvdecH264MasterDisParameters
 * \brief structure to store Mastering Display Colour Volume SEI parameters
 * \ingroup h264_group
 */
struct esclvdecH264MasterDisParameters
{
    uint16_t display_primaries_x[3];
    uint16_t display_primaries_y[3];
    uint16_t white_point_x;
    uint16_t white_point_y;
    uint32_t max_display_mastering_luminance;
    uint32_t min_display_mastering_luminance;
};

/**
 * \struct esclvdecH264LightLevelParameters
 * \brief structure to store Content Light Level Info SEI parameters
 * \ingroup h264_group
 */
struct esclvdecH264LightLevelParameters
{
    uint16_t max_content_light_level;
    uint16_t max_pic_average_light_level;
};

struct esclvdecH264SEIParameters
{
    esclvdecH264MasterDisParameters masterdis_param;
    esclvdecH264LightLevelParameters light_param;
};

struct esclvdecDecSEIParameters
{
    bool hevc_sei_valid;
    esclvdecHevcSEIParameters hevc; /**< SEI parameters of HEVC */
    bool h264_sei_valid;
    esclvdecH264SEIParameters h264; /**< SEI parameters of H264 */
};

struct esclvdecDecHwConfig
{
    uint32_t mpeg4_support;
    uint32_t custom_mpeg4_support;
    uint32_t h264_support;
    uint32_t vc1_support;
    uint32_t mpeg2_support;
    uint32_t jpeg_support;
    uint32_t jpeg_prog_support;
    uint32_t max_dec_pic_width;
    uint32_t max_dec_pic_height;
    uint32_t pp_support;
    uint32_t pp_config;
    uint32_t max_pp_out_pic_width;
    uint32_t sorenson_spark_support;
    uint32_t ref_buf_support;
    uint32_t tiled_mode_support;
    uint32_t vp6_support;
    uint32_t vp7_support;
    uint32_t vp8_support;
    uint32_t vp9_support;
    uint32_t avs_support;
    uint32_t jpeg_esupport;
    uint32_t rv_support;
    uint32_t mvc_support;
    uint32_t webp_support;
    uint32_t ec_support;
    uint32_t stride_support;
    uint32_t field_dpb_support;
    uint32_t avs_plus_support;
    uint32_t addr64_support;
    uint32_t hevc_support;
    uint32_t av1_support;
    uint32_t double_buffer_support;
    uint32_t hevc_main10_support;
    uint32_t vp9_10bit_support;
    uint32_t ds_support;
    uint32_t rfc_support;
    uint32_t ring_buffer_support;
    uint32_t fmt_p010_support;
    uint32_t fmt_customer1_support;
    uint32_t mrb_prefetch;
};

/**
 * \struct esclvdecDecApiVersion
 * \brief API Version information. \n
 * \ingroup common_group
 */
struct esclvdecDecApiVersion
{
    uint32_t major;           /**< API major version */
    uint32_t minor;           /**< API minor version */
};

/**
 * \struct esclvdecDecSwHwBuild
 * \brief Sw and Hw Build information. \n
 * \ingroup common_group
 */
struct esclvdecDecSwHwBuild
{
    uint32_t sw_build;
    uint32_t hw_build;
    esclvdecDecHwConfig hw_config[4];
};

struct esclvdecCrop
{
    uint32_t enabled; /**< whether cropping is enabled */
    uint32_t set_by_user; /**< cropping set by user, use this variable to record
                          whether user set crop.*/
    uint32_t x;        /**< cropping start x */
    uint32_t y;        /**< cropping start y */
    uint32_t width;    /**< cropping width */
    uint32_t height;   /**< cropping height */
};

struct esclvdecCrop2
{
    uint32_t enabled;
    uint32_t x;        /**< cropping start x */
    uint32_t y;        /**< cropping start y */
    uint32_t width;    /**< cropping width */
    uint32_t height;   /**< cropping height */
};

struct esclvdecScale
{
    uint32_t enabled;  /**< whether scaling is enabled */
    uint32_t scale_by_ratio;   /**< scaling output size will be calculated by ratio */
    uint32_t ratio_x;  /**< 0 indicate flexiable mode, or 1/2/4/8 indicate ratio */
    uint32_t ratio_y;  /**< 0 indicate flexiable mode, or 1/2/4/8 indicate ratio */
    uint32_t width;    /**< scaled output width */
    uint32_t height;   /**< scaled output height */
};

/**
 * \struct esclvdecPpUnitConfig
 * \brief A structure containing PP unit configuration information.
 * \ingroup common_group
 */
struct esclvdecPpUnitConfig
{
    uint32_t enabled;    /**< PP unit enabled */
    uint32_t tiled_e;    /**< PP unit tiled4x4 output enabled */
    uint32_t rgb;        /**< RGB output enabled */
    uint32_t rgb_planar; /**< RGB output planar output enabled */
    uint32_t cr_first;   /**< CrCb instead of CbCr */
    uint32_t shaper_enabled; /**< Hardware shaper feature enabled */
    uint32_t shaper_no_pad;  /**< shaper no need to pad for frame right edge */
    uint32_t dec400_enabled; /**< sw control shaper and dec400  */
    uint32_t planar;     /**< Planar output */
    esclvdecDecPicAlignment align;  /**< pp output alignment */
    uint32_t ystride; /**< Stride for Y plane : If set to 0, software should use the stride calculated 
                    from software. When not 0, software should check the validation of the value. */
    uint32_t cstride; /**< Stride for C plane : If set to 0, software should use the stride calculated 
                    from software. When not 0, software should check the validation of the value. */
    esclvdecCrop crop;
    esclvdecCrop2 crop2;
    esclvdecScale scale;
    uint32_t monochrome; /**< PP output monochrome (luma only) for YUV output */
    uint32_t out_p010; /**< P010 output with the  MSB bits valid */
    uint32_t out_1010; /**< Each channel(Y,U,V) use 10bits */
    uint32_t out_I010; /**< P010 output with the low 10bits valid */
    uint32_t out_L010; /**< 010 output with the LSB bits valid. */
    uint32_t out_p012; /**< P012 output with the  MSB bits valid */
    uint32_t out_I012; /**< Each channel(Y,U,V) use 12bits */
    uint32_t out_be; /**< absolute */
    uint32_t out_cut_8bits; /**< Force 8 bit output */
    uint32_t video_range;  /**< 1 - full range, 0 - limited range */
    uint32_t range_max; /**< limited max to video range */
    uint32_t range_min; /**< limited min to video range */
    uint32_t rgb_format;
    uint32_t rgb_stan;
    uint32_t rgb_alpha;  /**< alpha channel value for RGB format */
    uint32_t pp_filter;  /**< selector for pp scale algorithm */
    uint32_t x_filter_param; /**< when select LANCZOS scale, set the horizontal window param */
    uint32_t y_filter_param; /**< when select LANCZOS scale, set the vertical window param */
    uint32_t afbc_mode; /**< when pp tile enable, set the tiled format */
    uint32_t src_sel_mode; /**< set the center point select algorithm for pp scale, default is DOWN ROUND */
    uint32_t pad_sel; /**< whether pad value is used for pp scale for the picture boundary */
    uint32_t pad_Y; /**< the luma pad value */
    uint32_t pad_U; /**< the Cb pad value */
    uint32_t pad_V; /**< the Cr pad value */
};

struct esclvdecDWLInitParam
{
    uint32_t client_type;
};

struct esclvdecDWLLinearMemAllocInfo
{
    uint32_t size;
    uint32_t mem_type;
    uint32_t nid;
};

struct esclvdecDWLLinearMem
{
    uintptr_t virtual_address; /**< Pointer to the virtual address of stream linear buffer */
    uint64_t bus_address; /**< Bus address of the linear buffer. The start of the stream
                           buffer must be in a 128-bit aligned position or accessible
                           for reading from the previous 128-bit aligned position. */
    uint32_t size;         /**< physical size (rounded to page multiple) */
    uint32_t logical_size;
    uint32_t mem_type;
    uint64_t fd_dma;
    uint32_t nid;
};

struct esclvdecDecParams
{
    uint32_t output_picture_endian;    /**< output picture endian : big or little endian. */
    uint32_t bus_burst_length;         /**< how long are the hardware data bursts; better left unchanged. */
    uint32_t latency_compensation;     /**< compensation for bus latency; values up to 63. */
    uint32_t clk_gate_decoder;         /**< clock is gated from decoder between images and for disabled codecs. */
    uint32_t data_discard;             /**< decoder data disgard : enable or disable. */
    uint32_t refbu_data_excess_max_pct;/**< reference buffer : adjust value. */
    uint32_t bus_width64bit_enable;    /**< bus width is 64bit. */
    uint32_t latency;                  /**< reference buffer : memory wait states for latency. */
    uint32_t non_seq_clk;              /**< reference buffer : memory wait states for non_seq_clk. */
    uint32_t seq_clk;                  /**< reference buffer : memory wait states for seq_clk. */
    int32_t apf_threshold_value;      /**< APF threshold value. */
    uint32_t service_merge_disable;    /**< service merge disable. */
    uint32_t axi_wr_outstand;          /**< AXI outstanding threshold for write. */
    uint32_t axi_rd_outstand;          /**< AXI outstanding threshold for read. */
};

/**
 * \struct esclvdecDecCropParams
 * \brief Cropping info.
 * \ingroup common_group
 */
struct esclvdecDecCropParams
{
    /** The left offset value */
    uint32_t crop_left_offset;
    /** The wigth value */
    uint32_t crop_out_width;
    /** The top offset value */
    uint32_t crop_top_offset;
    /** The height value */
    uint32_t crop_out_height;
};

struct esclvdecDecOutput
{
    uintptr_t strm_curr_pos;
    uint64_t strm_curr_bus_address;
    uint32_t data_left;
    uintptr_t strm_buff;
    uint64_t strm_buff_bus_address;
    uint32_t buff_size;
};

/**
 * \struct esclvdecDecInputParameters
 * \brief Decoder input data
 * \ingroup common_group
 */
struct esclvdecDecInputParameters
{
    uint32_t pic_id; /**< picture decoded id */
    uintptr_t p_user_data; /**< potint to user data */
    uintptr_t stream; /**< point to input stream */
    uint32_t strm_len; /**< input stream length */
    uint64_t stream_bus_address; /**< input stream bus address */
    esclvdecDWLLinearMem stream_buffer;  /**< input stream buffer */
    uint32_t buffer_size; /**< input stream buffer size */
    uint32_t dec_image_type;   /**< Full image or Thumbnail to be decoded */
    uint32_t slice_mb_set; /**< slice mode: mcu rows to decode */
    uint32_t low_latency; /**< Indicates whether low latency is available */
    uint32_t frame_len; /**< The frame date length */
    uint32_t pic_decoded; /**< flag : this pic has been decoded. */
    uint32_t exit_send_thread; /**< flag : exit the send thread. */
    uint32_t slice_height;     /**< height of WebP slice, unused for other formats */
    esclvdecDecSkipFrameMode skip_frame;
    esclvdecDWLLinearMem picture_buffer_y;    /**< luma output address ==> if user allocated */
    esclvdecDWLLinearMem picture_buffer_cb_cr; /**< chroma output address ==> if user allocated */
    esclvdecDWLLinearMem picture_buffer_cr; /**< chroma output address ==> if user allocated */
};

/**
 * \brief Stream consumed callback prototype \n
 * This callback is invoked by the decoder to notify the application that 
 * a stream buffer was fully processed and can be reused.
 * \param stream base address of a buffer that was set as input when
 *                calling xxxDecDecode().
 * \param p_user_data application provided pointer to some private data.
 *                  This is set at decoder initialization time.
 * \sa esclvdecDecMCConfig;
 * \ingroup common_group
 */
/** unsupport callback */
/**
 * \struct struct esclvdecDecMCConfig
 * \brief Multicore decoder init configuration. \n
 * \ingroup common_group
 */
struct esclvdecDecMCConfig
{
    /** Indicates whether multicore is available */
    uint32_t mc_enable;
    uintptr_t stream_consumed_callback;
};

/**
 * \struct struct esclvdecDecInitConfig
 * \brief Decoder initialization params, will be used in esclvdecInit.
 * \ingroup common_group
 */
struct esclvdecDecInitConfig
{
    uintptr_t dwl_inst;                       /**< struct DWL instance. */
    esclvdecDecCodec codec;                         /**< Specifies the decoder types. */
    uint32_t disable_picture_reordering;         /**< Output in decoding order. */
    esclvdecDecErrorHandling error_handling;    /**< Error handling mode. */
    uint32_t use_video_compressor;               /**< Indicates whether RFC is available */
    uint32_t use_ringbuffer;                     /**< Indicates whether ringbuffer is available */
    uint32_t full_stream_mode;                   /**< Specifies the decoder input mode is full stream mode. */
    esclvdecDecDecoderMode decoder_mode;        /**< Decoder working mode. */
    uint32_t guard_size;                         /**< The minimum difference between minimum buffers number and allocated 
                                            buffers number that will force to return HDRS_RDY even buffers 
                                            number/size are sufficient for new sequence. */
    uint32_t use_adaptive_buffers;               /**< When sequence changes, if old output buffers (number/size) 
                                            are sufficient for new sequence, old buffers will be used 
                                            instead of reallocating output buffer. */
    uint32_t num_frame_buffers;
    uint32_t rlc_mode;                           /**< Indicates whether rlc mode is available(H264/MPEG4 only) */
    esclvdecDecErrorConcealment concealment_mode;   /**< Error concealment mode. */
    uint32_t max_num_pics_to_decode;                 /**< limits the decoding to N pictures. 0 for unlimited. */
    esclvdecDecMCConfig mc_cfg;                     /**< Multicore decoder init configuration. */
    esclvdecDecSkipFrameMode skip_frame;            /**< Skip some frames when decoding. */
    uint32_t auxinfo;                                /**< qp_output_enable */
    esclvdecDecInputFormat dec_format;              /**< Format of the input sequence(vp8/mpeg4 only). */
    uint32_t mvc;                                    /**< Indicates whether mvc stream */
    uint32_t nid;
};

/**
 * \struct struct esclvdecDecConfig
 * \brief Decoder setting params, will be used in esclvdecSetInfo.
 * \ingroup common_group
 */
struct esclvdecDecConfig
{
    esclvdecPpUnitConfig ppu_cfg[6]; /**< A structure containing PP unit configuration information. */
    esclvdecDecPicAlignment align; /**< Stride alignment: aligned to 1/8/16/.../512 bytes. */
    uint32_t hw_conceal; /**< Use Hw error concealment(only hevc/h264/av1).*/
    uint32_t disable_slice; /**< disable slice mode(only hevc/h264). */
    uint32_t dec_image_type; /**< decode image type : JPEGDEC_IMAGE or JPEGDEC_THUMBNAIL */
    uint32_t nid;
};

/**
 * \struct struct esclvdecDecSequenceInfo
 * \brief Video sequence information.
 * \ingroup common_group
 */
struct esclvdecDecSequenceInfo
{
    uint32_t pic_width;                    /**< decoded picture width in pixels */
    uint32_t pic_height;                   /**< decoded picture height in pixels */
    esclvdecDecCropParams crop_params; /**< Cropping parameters for the picture */
    esclvdecDecVideoRange video_range;   /**< YUV sample video range */
    uint32_t matrix_coefficients; /**< matrix coefficients RGB->YUV conversion */
    uint32_t bit_depth_luma;    /**< Bit depth of stored picture for luma */
    uint32_t bit_depth_chroma; /**< Bit depth of stored picture for chroma */
    uint32_t pic_stride;       /**< Byte width of the picture as stored in memory for luma */
    uint32_t pic_stride_ch;    /**< Byte width of the picture as stored in memory for chroma */
    esclvdecDecDpbMode dpb_mode;             /**< DPB mode : frame, or field interlaced */
    esclvdecDecPictureFormat output_format; /**< Format of the output picture */
    esclvdecDecPictureFormat output_format_thumb; /**< Format of the output picture in thumbnail */
    uint32_t coding_mode; /**< JPEG code type :
                         * JPEG_BASELINE
                         * JPEG_PROGRESSIVE
                         * JPEG_NONINTERLEAVED
                         */
    uint32_t coding_mode_thumb;    /**< JPEG code type in thumbnail :
                                 * JPEG_BASELINE
                                 * JPEG_PROGRESSIVE
                                 * JPEG_NONINTERLEAVED
                                 */
    uint32_t thumbnail_type;  /**< Thumbnail exist or not or not supported */
    uint32_t pp_enabled; /**< post process is available */
    uint32_t h264_base_mode;    /**< baseline profile, only H264 */
    esclvdecDecInputParameters jpeg_input_info; /**< Decoder input data(Only jpeg in here) */
    uint32_t img_max_dec_width; /**< Maximum supported decoding width for jpeg */
    uint32_t img_max_dec_height; /**< Maximum supported decoding height for jpeg */
};

struct esclvdecDecPictureInfo
{
    esclvdecDecPictureFormat format;
    uint32_t cycles_per_mb;
    uint32_t pic_id;
    uint32_t decode_id;
};

struct esclvdecDecPicture
{
    esclvdecDWLLinearMem luma;
    esclvdecDecPictureInfo picture_info;
    esclvdecDecSEIParameters sei_param;
    uint32_t pic_width;
    uint32_t pic_height;
    uint32_t pic_stride;
    uint32_t pic_stride_ch;
};

/**
 * \struct struct esclvdecDecPicturePpu
 * \brief Structure to carry information about decoded pictures for 'n' pp channal.
 * \ingroup common_group
 */
struct esclvdecDecPicturePpu
{
    esclvdecDecPicture pictures[6];
};

/**
 * \struct struct DecBufferInfo
 * \brief A structure containing the reference buffer information
 * \ingroup common_group
 */
struct esclvdecDecBufferInfo
{
    uint32_t next_buf_size; /**< extra buffer size */
    uint32_t buf_num; /**< extra buffer number */
    uint32_t add_extra_ext_buf; /**< number of extra buffer has been added */
    esclvdecDWLLinearMem buf_to_free; /**< the initialized buffer */
};

static const uint8_t kVdecIfId = 3;
static const uint8_t kesclvdecDWLInitId = 1;
static const uint8_t kesclvdecDWLReleaseId = 2;
static const uint8_t kesclvdecDWLReadAsicCoreCountId = 3;
static const uint8_t kesclvdecDWLMallocLinearId = 4;
static const uint8_t kesclvdecDWLFreeLinearId = 5;
static const uint8_t kesclvdecDWLmemsetId = 6;
static const uint8_t kesclvdecVCDecMCGetCoreCountId = 7;
static const uint8_t kesclvdecVCDecInitId = 8;
static const uint8_t kesclvdecVCDecGetInfoId = 9;
static const uint8_t kesclvdecVCDecSetInfoId = 10;
static const uint8_t kesclvdecVCDecDecodeId = 11;
static const uint8_t kesclvdecVCDecNextPictureId = 12;
static const uint8_t kesclvdecVCDecPictureConsumedId = 13;
static const uint8_t kesclvdecVCDecEndOfStreamId = 14;
static const uint8_t kesclvdecVCDecGetBufferInfoId = 15;
static const uint8_t kesclvdecVCDecAddBufferId = 16;
static const uint8_t kesclvdecVCDecAbortId = 17;
static const uint8_t kesclvdecVCDecAbortAfterId = 18;
static const uint8_t kesclvdecVCDecReleaseId = 19;
static const uint8_t kesclvdecUpdateInfoReportId = 20;
#endif // ESRPC_TYPE_DEFINITIONS_VDEC

#if defined(__cplusplus)
}
#endif

#endif // _VDEC_VDECIF_COMMON_H_
